/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *   Project name:      Biped Locomotion                        *
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: biped_gnuplot.hpp,v 1.5 2004/08/16 21:37:29 s0343981 Exp $
 ****************************************************************/
/**
 * \file: biped_gnuplot.hpp
 * \brief class GNUPlot definition.
 */

#ifndef __BIPED_GNUPLOT_HPP__
#define __BIPED_GNUPLOT_HPP__

#include <string>
#include <vector>
#include <list>

#define GNUPLOT_MAX_TMP_FILES    64
#define GNUPLOT_CMD_SIZE         1024

using namespace std;

// =============================================================================
//  Class: GNUPlot                                                             
/// \brief A C++ interface to gnuplot program.
/// Details: This is a direct translation from the C interface written by 
/// N. Devillard (http://ndevilla.free.fr/gnuplot/).
// =============================================================================
class GNUPlot {
public:
	GNUPlot();
	GNUPlot(const string & style);
	GNUPlot(const string & title, const string & style, 
			const string & xLabel, const string & yLabel, 
			const vector< double > & x, const vector< double > & y);
	GNUPlot(const string & title, const string & style,
			const string & xLabel, const string & yLabel,
			const vector< double > & x);
	~GNUPlot();

	// Send a command to gnuplot:
	void cmd(const char * cmdStr, ...);

	// Set line style:
	void setStyle(const string & style);

	// Set y and x axis labels:
	void setYLabel(const string & label) { setLabel(label, "ylabel"); }
	void setXLabel(const string & label) { setLabel(label, "xlabel"); }

	// Plot a single vector of data:
	void plotXData(const vector< double > & d, const string & title, 
				   const string & prefix = "");
	// Plot x,y pairs:
	void plotXYData(const vector< double > & x, const vector<double> & y,
					const string & title, const string & prefix = "");

	// Plot an equation of the form: y = ax + b:
	void plotSlope(double a, double b, const string & title);

	// plot an equation supplied as a string
	void plotEquation(const string & equation, const string & title);

	// if multiple plots are present it will clear the plot area
	void resetPlot(void);

	bool isValid(void) const { return isValid_; }

private:						// methods
	void init();
	bool getProgramPath(const string & pname);
	void setLabel(const string & label, const string & labelTypeStr);
	void plotVectorData(bool isPairs, const vector< double > & x, 
						const vector< double > & y, const string & title,
						const string & prefix = "");

private:						// members
	FILE * gnucmd_;
	string plotStyle_;
	list< string > tmpFilenames_;
	uint plotsNum_;
	bool isValid_;
};

#endif	// __BIPED_GNUPLOT_HPP__
