/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *   Project name:      Biped Locomotion                        *
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: biped_link.hpp,v 1.5 2004/08/06 18:02:43 s0343981 Exp $
 ****************************************************************/
/**
 * \file: biped_link.hpp
 * \brief class Link definition.
 */

#ifndef __BIPED_LINK_HPP__
#define __BIPED_LINK_HPP__

#include <ode/common.h>
#include <newmat/newmat.h>

// =============================================================================
//  Class: Link                                                          
/// \brief class definition of a robot's link (rigid particle).
// =============================================================================
class Link {
 public:						// methods
	Link() : m_() , r_(3) , p_(3) , a_(3) , w_(3) , I_(3, 3) { }
	Link(const dReal & m, const ColumnVector & r, const ColumnVector & p,
		 const ColumnVector & a, const ColumnVector & w, const Matrix & I) : 
		m_(m) , r_(r) , p_(p) , a_(a) , w_(w) , I_(I) { }
	Link(const Link & other) : m_(other.m_) , r_(other.r_) , p_(other.p_) ,  
							   a_(other.a_) , w_(other.w_) , I_(other.I_) { }
	Link & operator=(const Link & other)
	{
		if (this == &other)	return *this;
		m_ = other.m_;
		r_ = other.r_;
		p_ = other.p_;
		a_ = other.a_;
		w_ = other.w_;
		I_ = other.I_;
		return *this;
	}

	dReal & m(void) { return m_; }
	const dReal & m(void) const { return m_; }
	ColumnVector & r(void) { return r_; }
	const ColumnVector & r(void) const { return r_; }
	ColumnVector & p(void) { return p_; }
	const ColumnVector & p(void) const { return p_; }
	ColumnVector & a(void) { return a_; }
	const ColumnVector & a(void) const { return a_; }
	ColumnVector & w(void) { return w_; }
	const ColumnVector & w(void) const { return w_; }
	Matrix & I(void) { return I_; }
	const Matrix & I(void) const { return I_; }

 private:
	// Mass of the link:
	dReal m_;				
	
	// Center-of-mass position w.r.t. link coordinates system:
	ColumnVector r_;				
	
	// Position vector:
	ColumnVector p_;	

	// Linear acceleration vector:
	ColumnVector a_;	
	
	// Angular velocity:
	ColumnVector w_;

	// Inertia matrix w.r.t. CoM and link coordinate system orientation:
	Matrix I_;			
};

#endif	// __BIPED_LINK_HPP__
