/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *   Biped Locomotion                                           
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: biped_logging.hpp,v 1.5 2004/08/06 18:02:43 s0343981 Exp $
 ****************************************************************/
/**
 * \file: biped_logging.h
 * \brief Biped humanoid project logging facility definition.
 * Description:
 */

#ifndef __BIPED_LOGGING_HPP__
#define __BIPED_LOGGING_HPP__

#include <OTC/text/string.hh>
#include <OTC/Logger.hh>

#define PRINT_VALUE(val) { cout << #val << " = " << val << endl; }
#define PRINT_DEBUG(msg) { cout << "DEBUG: " << msg << endl; }
#define PRINT_VECTOR3(V) { cout << #V << " = [" << V[0] << "," << V[1]	\
								<< "," << V[2] << "]" << endl; }
#define PRINT_VECTOR4(V) { cout << #V << " = [" << V[0] << "," << V[1]	\
								<< "," << V[2] << "," << V[3] << "]" << endl; }
#define PRINT_VECTOR7(V) { cout << #V << " = ["	<< V[0] << "," << V[1]	\
								<< "," << V[2] << "," << V[3] << "," << V[4] \
								<< "," << V[5] << "," << V[6] << "]" << endl; }
#define PRINT_MATRIX3(V) { cout << #V << " = ["	<< V[0] << "," << V[1] << "," \
								<< V[2] << "," << V[3] << "," << V[4] << "," \
								<< V[5] << "," << V[6] << "," << V[7] << "," \
								<< V[8] << "]" << endl; }

#define BIPED_LOG_FILE  "/usr/local/webots/common/biped_log%p.%h-%d-%m-%y.log"

// Log level adaptation to OTC:
#define BIPED_EMERGENCY OTCLIB_LOG_EMERGENCY
#define BIPED_ALERT     OTCLIB_LOG_ALERT
#define BIPED_CRITICAL  OTCLIB_LOG_CRITICAL
#define BIPED_ERROR     OTCLIB_LOG_ERROR
#define BIPED_WARNING   OTCLIB_LOG_WARNING
#define BIPED_NOTICE    OTCLIB_LOG_NOTICE
#define BIPED_INFO      OTCLIB_LOG_INFO
#define BIPED_DEBUG     OTCLIB_LOG_DEBUG

typedef OTC_LogLevel BIPED_LogLevel; 

// =============================================================================
//  Class: BIPED_Logger                                                        
/// \brief Logger which handles messages for biped humanoid robots. 
/// Details: 
// =============================================================================
class BIPED_Logger : public OTC_Logger {
public:
	// Creates a logger which accumulates messages which have been sent to
	// <theChannel>. Note that it isn't valid to try and capture messages sent 
	// to the empty channel.
	BIPED_Logger(char const * channel) : OTC_Logger() , channel_(channel) { }
	~BIPED_Logger() { }

	/// Returns the channel of interest.
    OTC_String const & channel() const 
	{ return channel_; }

	/// Returns the accumulated messages. There will only be a newline character
	/// on the end of the message if one was part of the end of the message. 
	/// Individual messages will be separated by a newline character when they
	/// are appended as they arrive.
    OTC_String const & messages() const
	{ return messages_; }

	// Deletes all currently accumulated messages.
    void clearLog()
	{ messages_.truncate(); }

protected:
	/// Adds a message of certain length to those which have already been
	/// accumulated if the pattern matches the channel. 
    void log(char const * channel, BIPED_LogLevel level, char const * msg, 
			 u_int len)
	{
		if (channel == 0 || *channel == EOS)
			return;
		if (channel_ == channel) {
			if (!messages_.isEmpty())
				messages_.append(EOL);
			messages_.append(msg, len);
		}
	}

private:
	// Disable copy and assignment:
	BIPED_Logger(BIPED_Logger const &);
    BIPED_Logger & operator=(BIPED_Logger const &);

    OTC_String channel_;		///< The channel of interest.
    OTC_String messages_;		///< The accumulated messages.
};

#endif	// __BIPED_LOGGING_HPP__
