/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *   Project name:      Biped Locomotion                        *
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: biped_supervisor.hpp,v 1.5 2004/08/16 21:37:29 s0343981 Exp $
 ****************************************************************/
/**
 * \file: biped_supervisor.hpp
 * \brief class Supervisor definition.
 */

#ifndef __BIPED_SUPERVISOR_HPP__
#define __BIPED_SUPERVISOR_HPP__

#define TIME_STEP        ((unsigned short) 32) // millisec
#define COM_IND_DEF      "COM"
#define ZMP_IND_DEF      "ZMP"
#define TIME_LABEL_COLOR 0xff0000 // red
#define COM_LABEL_COLOR  0x000000 // black
#define ZMP_LABEL_COLOR  0xdd00ff // light purple
#define TIME_LABEL_X     0.75
#define TIME_LABEL_Y     0.01
#define COM_LABEL_X      0.01
#define COM_LABEL_Y      0.01
#define ZMP_LABEL_X      0.01
#define ZMP_LABEL_Y      0.05
#define LABEL_SIZE       0.04
#define SNAPSHOT_INTERVAL  ((float) 0.512) // sec

// =============================================================================
//  Class: Supervisor                                                          
/// \brief C++ definition of a biped humanoid experiment supervisor in Webots.
// =============================================================================
class Supervisor {
public:						// definitions
	typedef enum { COM_ID = 0, ZMP_ID, TIME_ID } LabelID;

public:						// methods
    Supervisor(DeviceTag receiver, const string & robotDEF);
    ~Supervisor() { }

	void run()
	{ 
		initialStep();
		for (;;) step(); 
	}
	void step();
	void initialStep();

private:						// methods
	void updateLabelAndIndicator(LabelID id, float vals[]);
	void resetRobotPosition();
	void takeSnapshot(float t);

private:						// members
	DeviceTag receiver_;
	NodeRef robotNode_;
	NodeRef comNode_;
	NodeRef zmpNode_;
	float robotInitPos_[7];
};

#endif	// __BIPED_SUPERVISOR_HPP__
