/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: biped_xml_handler.hpp,v 1.8 2004/08/15 20:04:42 s0343981 Exp $
 ****************************************************************/
/**
 * \file: biped_xml_handler.hpp
 * \brief class XMLHandler definition.
 */

#ifndef __BIPED_XML_HANDLER_HPP__
#define __BIPED_XML_HANDLER_HPP__

#include <string>
#include <vector>
#include <map>
#include <ode/common.h>
#include <SAX/helpers/DefaultHandler.h>

using std::string;

class Matrix;

typedef map< string, Matrix > MotionSequencesMap;

// =============================================================================
//  Class: XMLHandler                                                          
/// \brief A simple XML handler class for HOAP-2. Intended for parsing motion 
/// joint trajectories sequences files.
// =============================================================================
class XMLHandler : public SAX::DefaultHandler {
public:							// methods
	XMLHandler(MotionSequencesMap & motionsMap) : 
		SAX::DefaultHandler() , motionsMap_(motionsMap) , 
		viaPoints_() , currElement_() , seqID_() , jointID_() , timeType_() ,
		jointStrings_() ,  jointNum_(0) , isNewSeq_(false) { }
	virtual ~XMLHandler() { }
	
	// DTDHandler
	virtual void unparsedEntityDecl(const string & name, 
									const string & publicId,
									const string & systemId,
									const string & notationName);
	
	// ContentHandler
	virtual void startDocument();
	virtual void endDocument();
	virtual void startElement(const string & namespaceURI, 
							  const string & localName, const string & qName, 
							  const AttributesT& atts);
	virtual void endElement(const string & namespaceURI, 
							const string & localName, const string & qName);
	virtual void characters(const string & ch);
	virtual void processingInstruction(const string & target, const string & data);
	virtual void skippedEntity(const string & name);

	// ErrorHandler
	virtual void warning(const SAX::SAXParseException & e);
	virtual void error(const SAX::SAXParseException & e);
	virtual void fatalError(const SAX::SAXParseException & e);

	const MotionSequencesMap & getMotions() const { return motionsMap_; }

private:						// methods
	// Disable copy, assignment and equality:
	XMLHandler(const XMLHandler &);
	XMLHandler & operator=(const XMLHandler &);
	bool operator==(const XMLHandler &);
	
	void handleJointSequence(const string & ch);
	void handleTimeDefinition(const string & ch);

private:						// members
	MotionSequencesMap & motionsMap_;
	Matrix viaPoints_;
    string currElement_;
	string seqID_;
    string jointID_;
    string timeType_;
    vector< string > jointStrings_;
    uint jointNum_;
	bool isNewSeq_;
};

#endif	// __BIPED_XML_HANDLER_HPP__

