/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: hoap2_joints.hpp,v 1.5 2004/08/06 18:02:50 s0343981 Exp $ 
 ****************************************************************/
/**
 * \file hoap2_joints.hpp 
 */

#ifndef __HOAP2_JOINTS_HPP__
#define __HOAP2_JOINTS_HPP__

#include <cstring>
#include <biped_exception.hpp>
#include <biped_joints.hpp>
#include <biped_namespace.hpp>

#define JOINTS_NUM         25
#define JOINT_MAX_FORCE    9	// in Newton

#define ANKLE_PITCH_MIN_FORCE 4	// in Newton
#define ANKLE_ROLL_MIN_FORCE  4 // in Newton
#define ANKLE_PITCH_MAXVAL    1.06465
#define ANKLE_PITCH_MINVAL    -1.06465
#define ANKLE_ROLL_MAXVAL     0.43633
#define ANKLE_ROLL_MINVAL     -0.43633

BIPED_BEGIN_NAMESPACE(HOAP2);

typedef enum { torso_pitch = 0,
			   left_hip_roll, left_hip_pitch, left_hip_yaw, 
			   left_knee_pitch, 
			   left_ankle_roll, left_ankle_pitch, 
			   left_shoulder_roll, left_shoulder_pitch, left_shoulder_yaw, 
			   left_elbow_pitch, 
			   left_wrist_roll,
			   neck_pitch, neck_yaw, 
			   right_hip_roll, right_hip_pitch, right_hip_yaw, 
			   right_knee_pitch, 
			   right_ankle_roll, right_ankle_pitch, 
			   right_shoulder_roll, right_shoulder_pitch, right_shoulder_yaw,
			   right_elbow_pitch,
			   right_wrist_roll } JointID;

#define battery_body (right_wrist_roll + 1)

class HOAP2_JointToStr : public JointToStr {
 public:
	const char * toString(int joint) 
	{
		TRY(HOAP2_JointToStr::toString);
		switch (joint) {
			case torso_pitch: return "body_joint_1";
			case left_hip_roll: return "lleg_joint_2";
			case left_hip_pitch: return "lleg_joint_3";
			case left_hip_yaw: return "lleg_joint_1";
			case left_knee_pitch: return "lleg_joint_4";
			case left_ankle_roll: return "lleg_joint_6";
			case left_ankle_pitch: return "lleg_joint_5";
			case left_shoulder_roll: return "larm_joint_2";
			case left_shoulder_pitch: return "larm_joint_1";
			case left_shoulder_yaw: return "larm_joint_3";
			case left_elbow_pitch: return "larm_joint_4";
			case left_wrist_roll: return "larm_joint_5";
			case neck_pitch: return "head_joint_1";
			case neck_yaw: return "head_joint_2";
			case right_hip_roll: return "rleg_joint_2";
			case right_hip_pitch: return "rleg_joint_3";
			case right_hip_yaw: return "rleg_joint_1";
			case right_knee_pitch: return "rleg_joint_4";
			case right_ankle_roll: return "rleg_joint_6";
			case right_ankle_pitch: return "rleg_joint_5";
			case right_shoulder_roll: return "rarm_joint_2";
			case right_shoulder_pitch: return "rarm_joint_1";
			case right_shoulder_yaw: return "rarm_joint_3";
			case right_elbow_pitch: return "rarm_joint_4";
			case right_wrist_roll: return "rarm_joint_5";
			default: 
				THROW_ERROR("No such joint!");
		}
		CATCH;
	}

	const char * toDEFString(int joint) 
	{
		TRY(HOAP2_JointToStr::toString);
		switch (joint) {
			case torso_pitch: return "back_1_0";
			case left_hip_roll: return "left_hip_2_0";
			case left_hip_pitch: return "left_hip_3_0";
			case left_hip_yaw: return "left_hip_1_0";
			case left_knee_pitch: return "left_knee_0";
			case left_ankle_roll: return "left_ankle_2_0";
			case left_ankle_pitch: return "left_ankle_1_0";
			case left_shoulder_roll: return "left_shoulder_2_0";
			case left_shoulder_pitch: return "left_shoulder_1_0";
			case left_shoulder_yaw: return "left_shoulder_3_0";
			case left_elbow_pitch: return "left_elbow_0";
			case left_wrist_roll: return "left_hand_1_0";
			case neck_pitch: return "neck_tilt_0";
			case neck_yaw: return "neck_0";
			case right_hip_roll: return "right_hip_2_0";
			case right_hip_pitch: return "right_hip_3_0";
			case right_hip_yaw: return "right_hip_1_0";
			case right_knee_pitch: return "right_knee_0";
			case right_ankle_roll: return "right_ankle_2_0";
			case right_ankle_pitch: return "right_ankle_1_0";
			case right_shoulder_roll: return "right_shoulder_2_0";
			case right_shoulder_pitch: return "right_shoulder_1_0";
			case right_shoulder_yaw: return "right_shoulder_3_0";
			case right_elbow_pitch: return "right_elbow_0";
			case right_wrist_roll: return "right_hand_1_0";
			case battery_body: return "Hoap2_0";
			default: 
				THROW_ERROR("No such DEF!");
		}
		CATCH;
	}

};

BIPED_END_NAMESPACE;

#endif	// __HOAP2_JOINTS_HPP__
