/* -*- C++ -*- */
/****************************************************************
 *                                                              *
 *   Project name:      Biped Locomotion                        *
 *                                                              *
 *   Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 ****************************************************************
 * $Id: hoap2_shmem.hpp,v 1.5 2004/08/06 18:02:50 s0343981 Exp $
 ****************************************************************/
/**
 * \file: hoap2_shmem.hpp
 * \brief Hoap2 shared memory definitions.
 */

#ifndef __HOAP2_SHMEM_HPP__
#define __HOAP2_SHMEM_HPP__

#include <CGAL/Cartesian.h>
#include <ode/mass.h>
#include <biped_sem.hpp>

#define SHM_DEFAULT_FILE_PERMS DEFAULT_FILE_PERMS

// Hoap-2 shared memory definitions:
#define DYN_SEM_KEY  ((key_t) 6001)
#define FOOT_SEM_KEY ((key_t) 6002)
#define DYN_SHM_KEY  ((key_t) 6003)
#define FOOT_SHM_KEY ((key_t) 6004)

#define NUM_RIGID_BODIES 26
#define VECTOR3_LEN 3
#define VECTOR4_LEN 4
#define MATRIX3_LEN 12
#define MATRIX4_LEN 16
#define CONTACT_NUM 4			// each foot has max. 4 contact points.

// =============================================================================
//  Struct: dynamicsInfo                                                              
/// \brief Dynamics information to be passed to controller.
// =============================================================================
typedef struct dynamicsInfo {
	// Constant Dynamics:
	dReal mass[NUM_RIGID_BODIES]; ///< masses of all bodies
	dVector4 massPos[NUM_RIGID_BODIES]; ///< mass positions of all bodies
	dMatrix3 inertia[NUM_RIGID_BODIES]; ///< inertia tensors of all bodies
	// Changing dynamics:
	dVector3 angVel[NUM_RIGID_BODIES]; ///< angular velocities of all bodies
	dVector3 bodyPos[NUM_RIGID_BODIES]; ///< body positions of all bodies
	dVector3 bodyAcc[NUM_RIGID_BODIES]; ///< body accelerations of all bodies
} DynamicsInfo;

typedef CGAL::Point_3< CGAL::Cartesian< dReal > > Point_3;

// =============================================================================
//  Struct: footGroundInfo                                                               
/// \brief Foot-Ground information to be passed to controller.
// =============================================================================
typedef struct footGroundInfo {
	dVector3 footGroundContacts[2 * CONTACT_NUM]; ///< foot-ground contact pts
} FootGroundInfo;

#endif	// __HOAP2_SHMEM_HPP__
