/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: gnuplot_test.cpp,v 1.3 2004/08/15 19:45:41 s0343981 Exp $ 
 ****************************************************************/
/**
 * \file gnuplot_test.cpp 
 * \brief A simple program to test gnuplot interface.
 */

// C++ includes:
#include <iostream>
#include <sstream>
#include <iomanip>
#include <string>
#include <vector>
// BIPED includes:
#include <biped_exception.hpp>
#include <biped_gnuplot.hpp>
#include <biped_logging.hpp>

#define SLEEP_LGTH 1
#define NPOINTS    50

using std::cout;
using std::endl;
using std::string;

int main()
{
	TRY(main);
    vector< double > x;
    vector< double > y;
    int i;

    cout << "*** example of gnuplot control through C++ ***" << endl;

    GNUPlot g1 = GNUPlot("lines");

    // Slopes:
    cout << "*** plotting slopes" << endl;
    cout << "y = x" << endl;
    g1.plotSlope(1.0, 0.0, "unity slope");
    sleep(SLEEP_LGTH);

    cout << "y = 2*x" << endl;
    g1.plotSlope(2.0, 0.0, "y=2x");
    sleep(SLEEP_LGTH);

    cout << "y = -x" << endl;
    g1.plotSlope(-1.0, 0.0, "y=-x");
    sleep(SLEEP_LGTH);

    // Equations:
    g1.resetPlot();
    cout << endl << endl << "*** various equations" << endl;
    cout << "y = sin(x)" << endl;
    g1.plotEquation("sin(x)", "sine");
    sleep(SLEEP_LGTH);

    cout << "y = log(x)" << endl;
    g1.plotEquation("log(x)", "logarithm");
    sleep(SLEEP_LGTH);

    cout << "y = sin(x) * cos(2*x)" << endl;
    g1.plotEquation("sin(x)*cos(2*x)", "sine product");
    sleep(SLEEP_LGTH);

    // Styles:
    g1.resetPlot();
    cout << endl << endl << "*** showing styles" << endl;
    cout << "sine in points" << endl;
    g1.setStyle("points");
    g1.plotEquation("sin(x)", "sine");
    sleep(SLEEP_LGTH);

    cout << "sine in impulses" << endl;
    g1.setStyle("impulses");
    g1.plotEquation("sin(x)", "sine");
    sleep(SLEEP_LGTH);

    cout << "sine in steps" << endl;
    g1.setStyle("steps");
    g1.plotEquation("sin(x)", "sine");
    sleep(SLEEP_LGTH);

    // User defined 1d and 2d point sets:
    cout << endl << endl << "*** user-defined lists of doubles" << endl;
    for (i = 0; i < NPOINTS; i++)
        x.push_back((double)i * (double)i);
    g1.resetPlot();
    g1.setStyle("impulses");
    g1.plotXData(x, "user-defined doubles");
    sleep(SLEEP_LGTH);

    cout << endl << endl << "*** user-defined lists of points" << endl;
    x.clear();
    for (i = 0; i < NPOINTS; i++) {
        x.push_back((double) i);
        y.push_back((double) i * (double) i);
	}
    g1.resetPlot();
    g1.setStyle("points");
    g1.plotXYData(x, y, "user-defined points");
    sleep(SLEEP_LGTH);

    // Multiple output screens:
    cout << endl << endl;
    cout << "*** multiple output windows" << endl;
    g1.resetPlot();
    g1.setStyle("lines");

    GNUPlot g2 = GNUPlot("lines");
    GNUPlot g3 = GNUPlot("lines");

    cout << "window 1: sin(x)" << endl;
    g1.plotEquation("sin(x)", "sin(x)");
    sleep(SLEEP_LGTH);

    cout << "window 2: x*sin(x)" << endl;
    g2.plotEquation("x*sin(x)", "x*sin(x)");
    sleep(SLEEP_LGTH);

    cout << "window 3: log(x)/x" << endl;
    g3.plotEquation("log(x)/x", "log(x)/x");
    sleep(SLEEP_LGTH);

    // This should fail:
	GNUPlot g4 = GNUPlot("lines");
	cout << "window 4: sin(x)/x" << endl;
	g4.plotEquation("sin(x)/x", "sin(x)/x");
	sleep(SLEEP_LGTH);
	
    cout << endl << "*** end of gnuplot example" << endl;
	CATCH_FINAL;
    return 0;
}
