/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: pinv_test.cpp,v 1.5 2004/08/29 11:09:48 s0343981 Exp $ 
 ****************************************************************/
/**
 * \file pinv_test.cpp 
 * \brief A simple program to test pseudo-inverse calculation.
 */

// C++ includes:
#include <iostream>
#include <sstream>
#include <iomanip>
#include <string>
#include <vector>
// Newmat includes:
#include <newmat/newmat.h>
#include <newmat/newmatio.h>
// BIPED includes:
#include <biped_exception.hpp>
#include <biped_logging.hpp>
#include <biped_util.hpp>

using std::cout;
using std::endl;
using std::string;

static float initMat[3][8] = 
{ { -0.373054, 0.125705, 0.0916561, 0.0584878, 0.675093, 0.0606021, -0.0621905, -0.0707749 },
  { 0.00281091, -0.00119487, 0.000644572, 0.00548419, -0.0286448, -0.0299595, -0.0205112, -0.0179453 },
  { -0.0211466, -0.00227112, 0.00558733, 0.0141214, 0.0338375, -0.000570822, 0.000307029, -0.0055974 } };
static float initExpected[3][8] = 
{ { 0, 0, 0, 0, 0, 0, 0, 0 }, 
  { 0, 0, 0, 0, 0, 0, 0, 0 }, 
  { 0, 0, 0, 0, 0, 0, 0, 0 } };

static vector< Matrix > matrices;
static vector< Matrix > expected;

void init() 
{
	TRY(init);
	for (uint i = 0; i < 3; i++) {
		Matrix M(2, 4);
		Matrix E(2, 4);
		M << initMat[i][0] << initMat[i][1] << initMat[i][2] << initMat[i][3]
		  << initMat[i][4] << initMat[i][5] << initMat[i][6] << initMat[i][7];
		E << initExpected[i][0] << initExpected[i][1] << initExpected[i][2] << initExpected[i][3]
		  << initExpected[i][4] << initExpected[i][5] << initExpected[i][6] << initExpected[i][7];

		matrices.push_back(M);
		expected.push_back(E);
	}
	CATCH;
}

// -----------------------------------------------------------------------------
//  Function: main
/// \brief The main program.
// -----------------------------------------------------------------------------
int main() 
{
	TRY(main);
	init();
	
	try {
      
		for (uint i = 0; i < matrices.size(); i++) {
			Matrix M = matrices[i];
			Matrix pinvM = getPseudoInverse(M);
			PRINT_VALUE(M);
			PRINT_VALUE(pinvM);
			PRINT_VALUE(expected[i]);
		}
		   // catch exceptions thrown by my programs
	} catch (Exception) { cout << Exception::what() << endl; }	

	CATCH_FINAL;
	return 0;
}
