/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: hoap2_supervisor.cpp,v 1.5 2004/08/06 18:03:05 s0343981 Exp $ 
 ****************************************************************/
/**
 * \file hoap2_supervisor.cpp 
 */

#include <device/receiver.h>
#include <device/robot.h>
#include <iostream>
#include <biped_exception.hpp>
#include <biped_logging.hpp>
#include <biped_supervisor.hpp>
#include <hoap2/hoap2_joints.hpp>

using std::cerr;
using std::endl;
using namespace HOAP2;

static DeviceTag receiver;

// -----------------------------------------------------------------------------
//  Function: reset
/// \brief The global reset() implementation for a Webots controller.
/// Details: Here the device handles are collected.
// -----------------------------------------------------------------------------
void reset(void) 
{
	TRY(reset);
	receiver = robot_get_device("receiver");
	if (receiver == 0) 
		cerr << "missing receiver" << endl;
	CATCH;
}

// -----------------------------------------------------------------------------
//  Function: main
/// \brief The main program which runs the hoap-2 supervisor.
// ----------------------------------------------------------------------------- 
int main() 
{
	TRY(main);
	robot_live(reset);
	receiver_enable(receiver, TIME_STEP);
	
	HOAP2_JointToStr j2str;
	Supervisor * supervisor = new Supervisor(receiver,
											 j2str.toDEFString(JOINTS_NUM));
	supervisor->run();

	delete supervisor;
	CATCH_FINAL;
	return 0;
}
