/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: qrio_supervisor.cpp,v 1.6 2004/08/29 11:10:07 s0343981 Exp $ 
 ****************************************************************/
/**
 * \file qrio_supervisor.cpp 
 */

#include <device/receiver.h>
#include <device/robot.h>
#include <iostream>
#include <biped_exception.hpp>
#include <biped_logging.hpp>
#include <biped_supervisor.hpp>
#include <qrio/qrio_joints.hpp>

using std::cerr;
using std::endl;
using namespace QRIO;

static DeviceTag receiver;

// -----------------------------------------------------------------------------
//  Function: reset
/// \brief The global reset() implementation for a Webots controller.
/// Details: Here the device handles are collected.
// -----------------------------------------------------------------------------
void reset(void) 
{
	TRY(reset);
	receiver = robot_get_device("receiver");
	if (receiver == 0) 
		cerr << "missing receiver" << endl;
	CATCH;
}

// -----------------------------------------------------------------------------
//  Function: main
/// \brief The main program which runs the Qrio supervisor.
// ----------------------------------------------------------------------------- 
int main() 
{
	TRY(main);
	robot_live(reset);
	receiver_enable(receiver, TIME_STEP);

	QRIO_JointInfo jInfo;
	Supervisor * supervisor = new Supervisor(receiver, 
											 jInfo.toDEFString(JOINTS_NUM));
	supervisor->run();

	delete supervisor;
	CATCH_FINAL;
	return 0;
}
