#include <hoap2_exception.hpp>
#include <hoap2_joints.hpp>
#include <newmat/newmat.h>
#include <iostream>
#include <cmath>

using namespace HOAP2;

static float translations[25][3] = 
{
	{ 0, 0, 0 },				// back_1_0
	{ 0, 0, 0 },				// left_hip_2_0
	{ 0, 0, 0 },				// left_hip_3_0
	{ 0.039, -0.055, -0.034 },	// left_hip_1_0
	{ 0, -0.1, 0 },				// left_knee_0
	{ 0, 0, 0 },				// left_ankle_2_0
	{ 0, -0.1, 0 },				// left_ankle_1_0
	{ 0, 0, 0 },				// left_shoulder_2_0
	{ 0.1, 0.09, -0.0315 },		// left_shoulder_1_0
	{ 0, 0, 0 },				// left_shoulder_3_0
	{ 0, -0.101, 0 },			// left_elbow_0
	{ 0, -0.1, 0 },				// left_hand_1_0
	{ 0, 0, 0 },				// neck_tilt_0
	{ 0, 0.09, -0.0365 },		// neck_0
	{ 0, 0, 0 },				// right_hip_2_0
	{ 0, 0, 0 },				// right_hip_3_0
	{ -0.039, -0.055, -0.034 },	// right_hip_1_0
	{ 0, -0.1, 0 },				// right_knee_0
	{ 0, 0, 0 },				// right_ankle_2_0
	{ 0, -0.1, 0 },				// right_ankle_1_0
	{ 0, 0, 0 },				// right_shoulder_2_0
	{ -0.1, 0.09, -0.0315 },	// right_shoulder_1_0
	{ 0, 0, 0 },				// right_shoulder_3_0
	{ 0, -0.101, 0 },			// right_elbow_0
	{ 0, -0.1, 0 }				// right_hand_1_0
};

static float rotations[25][4] = 
{
	{ -1, 0, 0, 0 },			// back_1_0
	{ 0, 0, 1, 0 },				// left_hip_2_0
	{ 1, 0, 0, 0 },				// left_hip_3_0
	{ 0, 1, 0, 0 },				// left_hip_1_0
	{ 1, 0, 0, 0 },				// left_knee_0
	{ 0, 0, 1, 0 },				// left_ankle_2_0
	{ 1, 0, 0, 0 },				// left_ankle_1_0
	{ 0, 0, 1, 0 },				// left_shoulder_2_0
	{ -1, 0, 0, 0 },			// left_shoulder_1_0
	{ 0, 1, 0, 0 },				// left_shoulder_3_0
	{ 1, 0, 0, 0 },				// left_elbow_0
	{ 0, 0, 1, 0 },				// left_hand_1_0
	{ 1, 0, 0, 0 },				// neck_tilt_0
	{ 0, 1, 0, 0 },				// neck_0
	{ 0, 0, 1, 0 },				// right_hip_2_0
	{ 1, 0, 0, 0 },				// right_hip_3_0
	{ 0, 1, 0, 0 },				// right_hip_1_0
	{ 1, 0, 0, 0 },				// right_knee_0
	{ 0, 0, 1, 0 },				// right_ankle_2_0
	{ 1, 0, 0, 0 },				// right_ankle_1_0
	{ 0, 0, 1, 0 },				// right_shoulder_2_0
	{ -1, 0, 0, 0 },			// right_shoulder_1_0
	{ 0, 1, 0, 0 },				// right_shoulder_3_0
	{ 1, 0, 0, 0 },				// right_elbow_0
	{ 0, 0, 1, 0 }				// right_hand_1_0
};

void calcDH(float rot[], float trans[])
{
	float gamma = rot[3];
	float x = rot[0];	float u = trans[0];
	float y = rot[1];	float v = trans[1];
	float z = rot[2];	float w = trans[2];
	float c = cos(gamma); float s = sin(gamma); float t = 1 - c;

	float tx2 = t * x * x; float ty2 = t * y * y; float tz2 = t * z * z;
	float txy = t * x * y; float txz = t * x * z; 
	float tyz = t * y * z;
	float sx = s * x; float sy = s * y; float sz = s * z; 
	
	Matrix R(4, 4);
	R = 0.0;
	R(1, 1) = tx2 + c;    R(1, 2) = txy + sz;   R(1, 3) = txz - sy;
	R(2, 1) = txy - sz;   R(2, 2) = ty2 + c;    R(2, 3) = tyz + sx;
	R(3, 1) = txz + sy;   R(3, 2) = tyz - sx;   R(3, 3) = tz2 + c;
	R(4, 4) = 1.0;
	
	Matrix T(4, 4);
	T = 0.0;
	T(1, 1) = 1.0; T(2, 2) = 1.0; T(3, 3) = 1.0; T(4, 4) = 1.0;
	T(1, 4) = u;   T(2, 4) = v;   T(3, 4) = w;

	Matrix RT = R * T;

	float d = RT(3, 4);
	float theta = ((RT(1, 1) != 0) ? acos(RT(1, 1)) : asin(RT(2, 1)));
	float alpha = ((RT(3, 3) != 0) ? acos(RT(3, 3)) : asin(RT(3, 2)));
	float a = ((cos(theta) != 0) ? (RT(1, 4) / cos(theta)) : (RT(2, 4) / sin(theta)));

	cout << "theta:\t\t" << theta << endl;
	cout << "d:\t\t"     << d     << endl;
	cout << "a:\t\t"     << a     << endl;
	cout << "alpha:\t\t" << alpha << endl;
}

int main()
{
	for (uint i = 0; i < JOINTS_NUM; i++) {
		cout << "Joint " << JointToStr::toDEFString(i) << ":\n";
		calcDH(rotations[i], translations[i]);
	}	
	return 0;
}
