package MotionTool.Interface;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.*;
import java.io.*;
import java.awt.image.*;
import java.lang.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Iterator;
import java.util.Enumeration;

import MotionTool.Interface.*;
import MotionTool.*;
import com.cyberbotics.webots.Controller;


public class MotionInterface extends JFrame {

    //The number of servos used
    int nbrJoints = MotionTool.GlobalDefinitions.nbrJoints;

    // The Dimensions of the planning Field
    public int maxTime = 500; //2000 * 32ms = a bit more than 1 min
    public int actualDrawingMotor = 0;

    //Global variables
    public int letsGo = 0;
    public int periodBeginning = 0;
    public int periodEnd = 499;
    public int timeAtTheBeginning = 0;
    public Motion actualMotion = new Motion();
    public Vector motionVector = new Vector();
    public int actualMotionIndex = 0;

    /**************************************************
     The components of the Interface
	***************************************************/

    //All the buttons of the interface
    public JButton jButtonGo = new JButton();
    public JButton jButtonDelete = new JButton();
    public JButton jButtonLoad = new JButton();
    public JButton jButtonSave = new JButton();
    public JButton jButtonNew = new JButton();
    public JButton jButtonChangeName = new JButton();
    public JButton jButtonDeleteMotion = new JButton();

    //The text Fields of the Interface
    public JLabel jLabelServoId = new JLabel();
    public JLabel jLabelServoValue = new JLabel();
    public JLabel jLabelServoFrameNumber = new JLabel();
    public JLabel jLabelServoId2 = new JLabel();
    public JLabel jLabelServoValue2 = new JLabel();
    public JLabel jLabelServoFrameNumber2 = new JLabel();
    public JLabel jLabelSmoothFactor = new JLabel();
    public JLabel jLabelDrawingRobot = new JLabel();
    public JLabel jLabelResultRobot = new JLabel();
    public JLabel jLabelDrawingBoard = new JLabel();
    public JLabel jLabelDrawingBoard2 = new JLabel();
    public JLabel jLabelResultBoard = new JLabel();
    public JLabel jLabelResultBoard2 = new JLabel();

    public JTextField jTextSmoothFactor = new JTextField();

    //The Label Fields of the Interface
    public JLabel jLabelMousePosX = new JLabel("Frame Number:");
    public JLabel jLabelMousePosY = new JLabel("Servo Value:");

    //Other Interface Elements
    public JCheckBox jCheckBoxManualMode = new JCheckBox();
    public Panel panelTimeBar = new Panel();
    private JList motionList;
    private DefaultListModel listModel;


    //The two drawing boards
    public Board drawingBoard = new Board(new BoardEventHandler(true), 
										  new BoardMotionEventHandler(true), 430);
    public Board resultBoard = new Board(new BoardEventHandler(false), 
										 new BoardMotionEventHandler(false), 170);

    // colors of the ScaleDrawings
    final int colorPinkLight = 0xffffddff;
    final int colorPink = 0xffffbbff;
    final int colorBlueLight = 0xffddddff;
    final int colorGreyLight = 0xffdddddd;
    final int colorSkyLight1 = 0xffaaaaff;
    final int colorSkyLight2 = 0xffb3ffff;
    final int colorSkyLight3 = 0xffceffff;

    public void initialize() 
	{
        int i;
        //by default the working period is 0-500
        periodBeginning = 0;
        periodEnd = drawingBoard.boardWidth;

        Motion temp = new Motion();
        temp.motionName = "NewMotion";
        motionVector.add(temp);

        /***************************************************************************
         * Setup the Graphical interface
         ***************************************************************************/

        // General
        this.getContentPane().setBackground(Color.lightGray);
        this.setLocale(java.util.Locale.getDefault());
        this.setJMenuBar(null);
        this.setTitle("MotionTool");
        this.getContentPane().setLayout(null);

        //Set Min Max Vaqlue for the drawing boards
        drawingBoard.setMinMaxBoardValuesFromServoLimits(
			GlobalDefinitions.servoMinValue[actualDrawingMotor], 
			GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
        resultBoard.setMinMaxBoardValuesFromServoLimits(
			GlobalDefinitions.servoMinValue[actualDrawingMotor], 
			GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
        // Erase the drawing Board and draw the scales
        drawingBoard.redrawBoard();
        //Add drawing board to interface
        this.getContentPane().add(drawingBoard.boardImageLabel);
        //Draw  the scales on the result board
        resultBoard.redrawBoard();
        //Add result board to interface
        this.getContentPane().add(resultBoard.boardImageLabel);

        /********************************************************
         ALL Buttons Definitions of the Interface
		*******************************************************/

        //Initialises and adds all the Buttons related to the
        //Robots servos (visble servos and actual Drawing servo
        initRobotButtons();

        //Go Button
        jButtonGo.setBackground(Color.green);
        jButtonGo.setBounds(new Rectangle(510, 30, 100, 34));
        jButtonGo.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonGo.setName("go");
        jButtonGo.setText("GO !");
        jButtonGo.setForeground(Color.black);
        jButtonGo.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonGo, null);

        jButtonDelete.setBounds(new Rectangle(510, 430, 100, 22));
        jButtonDelete.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonDelete.setText("Delete");
        jButtonDelete.setName("Delete");
        jButtonDelete.setEnabled(false);
        jButtonDelete.setForeground(Color.black);
        jButtonDelete.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonDelete, null);

        //Load Button                       510, 300, 100, 22
        jButtonLoad.setBounds(new Rectangle(510, 80, 100, 22));
        jButtonLoad.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonLoad.setText("Load File...");
        jButtonLoad.setName("load");
        jButtonLoad.setEnabled(true);
        jButtonLoad.setForeground(Color.black);
        jButtonLoad.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonLoad, null);

        //Save Button
        jButtonSave.setBounds(new Rectangle(510, 110, 100, 22));
        jButtonSave.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonSave.setText("Save File...");
        jButtonSave.setName("save");
        jButtonSave.setEnabled(true);
        jButtonSave.setForeground(Color.black);
        jButtonSave.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonSave, null);


        jButtonNew.setBounds(new Rectangle(510, 235, 100, 22));
        jButtonNew.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonNew.setText("New");
        jButtonNew.setName("newMotion");
        jButtonNew.setEnabled(true);
        jButtonNew.setForeground(Color.black);
        jButtonNew.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonNew, null);

        jButtonChangeName.setBounds(new Rectangle(510, 260, 100, 22));
        jButtonChangeName.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonChangeName.setText("Rename...");
        jButtonChangeName.setName("editMotion");
        jButtonChangeName.setEnabled(true);
        jButtonChangeName.setForeground(Color.black);
        jButtonChangeName.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonChangeName, null);

        jButtonDeleteMotion.setBounds(new Rectangle(510, 285, 100, 22));
        jButtonDeleteMotion.setFont(new java.awt.Font("Dialog", 1, 12));
        jButtonDeleteMotion.setText("Delete...");
        jButtonDeleteMotion.setName("deleteMotion");
        jButtonDeleteMotion.setEnabled(true);
        jButtonDeleteMotion.setForeground(Color.black);
        jButtonDeleteMotion.addMouseListener(new jButton_mouseAdapter(this));
        this.getContentPane().add(jButtonDeleteMotion, null);

        /********************************************************
         ALL Text Fields Definitions of the Interface
		*******************************************************/

        jTextSmoothFactor.setName("SmoothFactor");
        jTextSmoothFactor.setText("0");
        jTextSmoothFactor.setBounds(new Rectangle(510, 335, 100, 22));
        jTextSmoothFactor.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jTextSmoothFactor, null);

        /********************************************************
         ALL Label Fields Definitions of the Interface
		*******************************************************/
        jLabelServoId.setName("ServoId");
        jLabelServoId.setBounds(new Rectangle(510, 478, 100, 22));
        jLabelServoId.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoId, null);

        jLabelServoId2.setName("ServoId2");
        jLabelServoId2.setBounds(new Rectangle(510, 498, 100, 22));
        jLabelServoId2.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoId2, null);

        jLabelServoValue.setName("ServoValue");
        jLabelServoValue.setBounds(new Rectangle(510, 528, 100, 22));
        jLabelServoValue.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoValue, null);

        jLabelServoValue2.setName("ServoValue2");
        jLabelServoValue2.setBounds(new Rectangle(510, 548, 100, 22));
        jLabelServoValue2.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoValue2, null);

        jLabelServoFrameNumber.setName("FrameNumber");
        jLabelServoFrameNumber.setBounds(new Rectangle(510, 578, 100, 22));
        jLabelServoFrameNumber.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoFrameNumber, null);

        jLabelServoFrameNumber2.setName("FrameNumber2");
        jLabelServoFrameNumber2.setBounds(new Rectangle(510, 598, 100, 22));
        jLabelServoFrameNumber2.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelServoFrameNumber2, null);

        jLabelSmoothFactor.setName("SmoothFactor");
        jLabelSmoothFactor.setText("Smooth Factor");
        jLabelSmoothFactor.setBounds(new Rectangle(510, 310, 100, 22));
        jLabelSmoothFactor.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelSmoothFactor, null);


        jLabelResultRobot.setText("Result Board (hide/show)");
        jLabelResultRobot.setBounds(new Rectangle(300, 0, 180, 22));
        jLabelResultRobot.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelResultRobot, null);

        jLabelDrawingRobot.setText("Drawing Board (select)");
        jLabelDrawingRobot.setBounds(new Rectangle(80, 0, 180, 22));
        jLabelDrawingRobot.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelDrawingRobot, null);

        jLabelDrawingBoard.setText("Drawing Board:");
        jLabelDrawingBoard.setBounds(new Rectangle(5, 405, 180, 22));
        jLabelDrawingBoard.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelDrawingBoard, null);

        jLabelDrawingBoard2.setText("back_1 (Button 0)");
        jLabelDrawingBoard2.setBounds(new Rectangle(190, 405, 180, 22));
        jLabelDrawingBoard2.addKeyListener(new jText_keyAdapter());
        //this.getContentPane().add(jLabelDrawingBoard2, null);

        jLabelResultBoard.setText("Result Board: Click on result Board Buttons to hide/show servos");
        jLabelResultBoard.setBounds(new Rectangle(5, 140, 500, 22));
        jLabelResultBoard.addKeyListener(new jText_keyAdapter());
        this.getContentPane().add(jLabelResultBoard, null);

        jLabelMousePosX.setBounds(new Rectangle(281, 405, 138, 21));
        this.getContentPane().add(jLabelMousePosX, null);

        jLabelMousePosY.setBounds(new Rectangle(411, 405, 138, 21));
        this.getContentPane().add(jLabelMousePosY, null);

//         jCheckBoxManualMode.setText("Manual Mode:");
// 		jCheckBoxManualMode.setBounds(510, 6, 71, 34);
// 		this.getContentPane().add(jCheckBoxManualMode, null);
		
        // alist
        listModel = new DefaultListModel();
        listModel.addElement("NewMotion");

        //Create the list and put it in a scroll pane.
        motionList = new JList(listModel);
        motionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        motionList.setSelectedIndex(0);
        //list.addListSelectionListener();
        motionList.setVisibleRowCount(2);
        motionList.addListSelectionListener(new motionListSelectionListener(this));
        JScrollPane listScrollPane = new JScrollPane(motionList);
        listScrollPane.setBounds(510, 170, 100, 60);
        this.getContentPane().add(listScrollPane, BorderLayout.CENTER);

        //The time Bar
        panelTimeBar.setBackground(Color.yellow);
        panelTimeBar.setCursor(null);
        panelTimeBar.setDropTarget(null);
        panelTimeBar.setForeground(Color.yellow);
        panelTimeBar.setVisible(false);
        panelTimeBar.setBounds(new Rectangle(2, 170, 2, 489));
        panelTimeBar.setFocusable(true);
        this.getContentPane().add(panelTimeBar, null);

        this.repaint();
    }

    public Motion getActualMotion() 
	{
//         Motion actual = new Motion();
// 		System.out.println("Fetching Motion.... ");
// 		Iterator it = actualMotion.motionDefinition.iterator();
				
// 		while (it.hasNext()) {
// 			int servoId = ((ServoKeyFrames) it.next()).servoId;
// 			actual.appendServoKeyFrames(actualMotion.getServoKeyFrames(servoId));
// 			System.out.println("Actual Motion is " + actual.toString());
// 		}
        return actualMotion;

    }

    public int loadFileToBoard(String fileName) {

        System.out.println("Loading file ....");
        //Use te File Parser to parse the file
        FileParser loadedFile = new FileParser(fileName, true, true);

        if (!loadedFile.hasParsedError) {
            //If the parser ended with no errors: Get the first defined Motion
            actualMotionIndex = 0;

            motionVector = loadedFile.getMotions();
            // add all the motion Names to the motion list
            listModel.removeAllElements();
            Iterator motIt = motionVector.iterator();
            while (motIt.hasNext()){
				Motion temp = (Motion)motIt.next();
				listModel.addElement(temp.motionName);
            }



            actualMotion = (Motion) motionVector.elementAt(actualMotionIndex);


            //Remove the Old Motion definitions from the result board
            resultBoard.removeAlleKeyFrames();
            //Copy all servoKeyFrame Definitions into the result board
            Iterator it = actualMotion.motionDefinition.iterator();
            ServoKeyFrames temp = new ServoKeyFrames(0);
            while (it.hasNext()) {
                temp = (ServoKeyFrames) it.next();
                resultBoard.setTransformedServoKeyFrames(temp, true);
            }
            //Set the actual drawing motor to the last entry of the file
            actualDrawingMotor = temp.servoId;
            //Copy the servo values for the actual servo to the result board
            drawingBoard.setTransformedServoKeyFrames(temp, false);


            //Unselect all Buttons and set all Servos to visible
            unselectAllButtons(actualDrawingMotor);

            //Set the servos limits to the board
            drawingBoard.setMinMaxBoardValuesFromServoLimits(
				GlobalDefinitions.servoMinValue[actualDrawingMotor], 
				GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
            resultBoard.setMinMaxBoardValuesFromServoLimits(
				GlobalDefinitions.servoMinValue[actualDrawingMotor], 
				GlobalDefinitions.servoMaxValue[actualDrawingMotor]);

            //Redraw booth boards
            drawingBoard.redrawBoard();
            resultBoard.redrawBoard();

            //Set the save text field to the actual name
            return 1;
        }
        return -1;
    }

    public int saveBoardToFile(String fileName) {

        System.out.println("Saving file ....");
        //Get the actual drawing and put it in the motion Vector
        ServoKeyFrames newDrawing = drawingBoard.getTransFormedServoKeyFrames(actualDrawingMotor);
        actualMotion.appendServoKeyFrames(newDrawing);
        actualMotion.updatenumberOfFramesInMotion();
        actualMotion.motionName = (String)listModel.elementAt(actualMotionIndex);

        //Write the Motion to the File
        XMLFileWriter writer = new XMLFileWriter();
        return writer.writeToFile(motionVector, fileName);
    }

    class jText_keyAdapter extends java.awt.event.KeyAdapter {

        public void keyPressed(KeyEvent e) {
            JTextField actualField = (JTextField) e.getSource();
            String textFieldName = actualField.getName();
            String textFieldValue = actualField.getText();


            System.out.println("Key Pressed :" + e.getKeyChar() + " comming from " + textFieldName);
            String str;
            //Hit the enter Key
            if (e.getKeyCode() == 10) {
                System.out.println("is Enter");

                //Load File Clicked
                if (textFieldName.equals("loadFileName")) {
                    loadFileToBoard(textFieldValue);
                    //Save file clicked
                } else if (textFieldName.equals("saveFileName")) {
                    saveBoardToFile(textFieldValue);

                }
            }
        }

    }

    class motionListSelectionListener implements ListSelectionListener {

        JFrame parentInterface;

        //This method is required by ListSelectionListener.
        public motionListSelectionListener(JFrame parent) {
            parentInterface = parent;

        }

        public void valueChanged(ListSelectionEvent e) {

            System.out.println("Listener works Index=" + motionList.getSelectedIndex());
            if (e.getValueIsAdjusting() == false) {


                if (motionList.getSelectedIndex() == -1) {
                    //No selection, disable fire button.
                    //fireButton.setEnabled(false);


                } else {
                    System.out.println("Clicked on motion with index: " + motionList.getSelectedIndex());

                    changeBoardsToNewMotion(motionList.getSelectedIndex(),true);
                    //actualMotion = (Motion) motionVector.elementAt(motionList.getSelectedIndex());
                    //Selection, enable the fire button.
                    //fireButton.setEnabled(true);
                }
            }
        }

    }


    public void changeBoardsToNewMotion(int newPosition, boolean saveFirst) {

        if (saveFirst){
			//save current view
			//Get the actual drawing and put it in the motion Vector
			ServoKeyFrames newDrawing = drawingBoard.getTransFormedServoKeyFrames(actualDrawingMotor);
			System.out.println("Saving Drawing...: " + newDrawing.toString());
			((Motion) motionVector.elementAt(actualMotionIndex)).appendServoKeyFrames(newDrawing);
			System.out.println("Motion saved is " + ((Motion) motionVector.elementAt(actualMotionIndex)));
        }

        //Change actual Motion
        actualMotionIndex = newPosition;
        actualMotion = (Motion) motionVector.elementAt(actualMotionIndex);
        //change to the servo slected
        actualDrawingMotor = ((ServoKeyFrames) actualMotion.motionDefinition.firstElement()).servoId;
        System.out.println("New Motion is " + actualMotion);
        unselectAllButtons(actualDrawingMotor);

        //Search in the actual Motion if the servo already has keyFrames
        ServoKeyFrames newKeyFrames = actualMotion.getServoKeyFrames(actualDrawingMotor);
        System.out.println("New KeyFrames Loaded from Motion " + actualMotion.motionName + 
						   ": " + newKeyFrames.toString());

        //Transform the new Motion's keyFrames to drawing Board values and replace the boards key Frames
        drawingBoard.setTransformedServoKeyFrames(newKeyFrames, false);
        resultBoard.setMotion(actualMotion);
        //System.out.println("Is the actual one : " + drawingBoard.boardKeyFrames.getServoKeyFrames(actualDrawingMotor).toString());
        //Set the servos limits to the board
        drawingBoard.setMinMaxBoardValuesFromServoLimits(
			GlobalDefinitions.servoMinValue[actualDrawingMotor], 
			GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
        resultBoard.setMinMaxBoardValuesFromServoLimits(
			GlobalDefinitions.servoMinValue[actualDrawingMotor], 
			GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
        drawingBoard.redrawBoard();
        resultBoard.redrawBoard();

        /*

        //Set global variables to new situation
        System.out.println("Motion Vector now has " + motionVector.size() + " elements; changing to position "+ position);
        actualMotion = (Motion) motionVector.elementAt(position);
        actualDrawingMotor = ((ServoKeyFrames) (actualMotion.motionDefinition.firstElement())).servoId;

        //resetButtons
        unselectAllButtons(actualDrawingMotor);
        actualMotion.motionName = jTextMotionName.getText();

        //set the new motion to the result board
        resultBoard.setMotion(actualMotion);
        //Set first KeyFrame servo
        //drawingBoard.removeAlleKeyFrames();
        drawingBoard.setTransformedServoKeyFrames((ServoKeyFrames) (actualMotion.motionDefinition.firstElement()), true);
        resultBoard.redrawBoard();
        drawingBoard.redrawBoard();
        */
    }


    class jButton_mouseAdapter extends java.awt.event.MouseAdapter {

        MotionInterface theInterface;

        public jButton_mouseAdapter(MotionInterface theInterface) {
            this.theInterface = theInterface;
        }


        public void mouseClicked(MouseEvent e) {

            JButton button = (JButton) e.getSource();
            String buttonName = button.getName();
            System.out.println("Clicked on button " + buttonName);

            if (buttonName == "Delete") {
                System.out.println("Clicked deleted");
                if (button.isEnabled()) {
                    System.out.println("Deleting at pos " + deletKeyFramePos);
                    button.setEnabled(false);
                    drawingBoard.removeKeyFrame(actualDrawingMotor, deletKeyFramePos);
                    resultBoard.removeKeyFrame(actualDrawingMotor, deletKeyFramePos);
                    theInterface.frameIsClicked = false;
                } else
                    System.out.println("Delete is  not actif");

            } else if (buttonName == "newMotion") {

                System.out.println("New Motion clicked");

                listModel.addElement("newMotion");

                ServoKeyFrames keyFrame = new ServoKeyFrames(0);
                Motion newMotion = new Motion();
                newMotion.appendServoKeyFrames(keyFrame);
                newMotion.motionName = "newMotion";
                //Add new Motion to Motion Vector
                motionVector.addElement(newMotion);

                changeBoardsToNewMotion(motionVector.size() - 1, true);
                motionList.setSelectedIndex(actualMotionIndex);
            } else if (buttonName == "go") {
                System.out.println("Clicked Go");
                if (letsGo == 0) {

                    jButtonGo.setBackground(Color.red);
                    jButtonGo.setText("STOP");
                    //Get the actual drawing and put it in the motion Vector
                    ServoKeyFrames newDrawing = drawingBoard.getTransFormedServoKeyFrames(actualDrawingMotor);
                    actualMotion.appendServoKeyFrames(newDrawing);
                    actualMotion.updatenumberOfFramesInMotion();
                    float smoothFactor = Float.parseFloat(jTextSmoothFactor.getText());
                    System.out.println(" Setting Smooth : " + smoothFactor);
                    actualMotion.setSmoothFactorMotionTransition(smoothFactor);

                    letsGo = 1;
                } else {
                    letsGo = 3;
                }
            } else if (buttonName == "load") {
                openFileChooser();
            } else if (buttonName == "save") {
                saveFileChooser();
            } else if (buttonName=="editMotion"){

                int pos = motionList.getMinSelectionIndex();
                if (pos >=0){
                    String inputValue = JOptionPane.showInputDialog("Please input a new Motion Name");
                    if (inputValue != null && !inputValue.equals("")){
						System.out.println("You have chosen a new motion Name : " + inputValue + " for position " + pos);
						listModel.setElementAt(inputValue,pos);
						((Motion)motionVector.elementAt(pos)).motionName = inputValue;
                    }
                }
            } else if (buttonName=="deleteMotion"){
				int pos = motionList.getMinSelectionIndex();
				String motionName = (String)listModel.elementAt(pos);


				Object[] options = {"Delete",
									"Cancel"};
				int cancel = JOptionPane.showOptionDialog(theInterface,
														  "Are you shure you want to remove Motion " + motionName,
														  "Remove Motion ?",
														  JOptionPane.YES_NO_OPTION,
														  JOptionPane.QUESTION_MESSAGE,
														  null, //don't use a custom Icon
														  options, //the titles of buttons
														  options[0]); //default button title

				//Overwrite the file
				if (cancel == 0){
					System.out.println("List size is " + motionList.getSize() + " Trying to erase position " + pos);
					listModel.remove(pos);
					motionVector.removeElementAt(pos);
					if (motionVector.size()< 1){
						// KeyFrame temp = new KeyFrame(0,0);
						listModel.addElement("NewMotion");
						ServoKeyFrames servoTemp= new ServoKeyFrames(0);
						Motion motTemp = new Motion();
						motTemp.appendServoKeyFrames(servoTemp);
						motTemp.motionName="NewMotion";
						motionVector.add(motTemp);
					}
					changeBoardsToNewMotion(0, false);
				}

				/*
				  String inputValue = JOptionPane.showConfirmDialog()
				  InputDialog("Please input a new Motion Name");
				  if (inputValue != null && !inputValue.equals("")){
				  System.out.println("You have chosen a new motion Name : " + inputValue + " for position " + pos);
				  listModel.setElementAt(inputValue,pos);
				  ((Motion)motionVector.elementAt(pos)).motionName = inputValue;
                */
			}
		}




        public void openFileChooser() {
			//Open File Chooser
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new XMLFilter());
			chooser.setCurrentDirectory(new File("/usr/local/webots/controllers/hoap2_motionTool"));
			int returnVal = chooser.showOpenDialog(theInterface);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				String fileName = chooser.getSelectedFile().getAbsolutePath();
				System.out.println("You chose to open this file: " + fileName);
				try {
					//Load file and create a corresponding Tokenizer
					FileReader File = new FileReader(fileName);
					if (loadFileToBoard(fileName) < 0) {
						JOptionPane.showMessageDialog(theInterface,
													  "There was an error Parsing the File. Please view the errors in the errorLog.txt / debugLog.txt files",
													  "Parser Error",
													  JOptionPane.ERROR_MESSAGE);
					}
				} catch (IOException ex) {
					System.out.println("Caught access file IOException: " + ex.toString());
					//Error: The file doesnt exist
					JOptionPane.showMessageDialog(theInterface,
												  "The File you've choosen dosn't exist",
												  "Error",
												  JOptionPane.ERROR_MESSAGE);
				} // end of try / catch for file acces errors
			}
		}

		public void saveFileChooser() {
			//Open File Chooser
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new XMLFilter());
			chooser.setCurrentDirectory(new File("/usr/local/webots/controllers/hoap2_motionTool"));
			int returnVal = chooser.showOpenDialog(theInterface);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				String fileName = chooser.getSelectedFile().getAbsolutePath();
				System.out.println("You chose to open this file: " + fileName);
				try {
					//Load file and create a corresponding Tokenizer
					FileReader File = new FileReader(fileName);
					Object[] options = {"Overwrite",
										"Cancel"};
					int cancel = JOptionPane.showOptionDialog(theInterface,
															  "The file you have chosen already exists, do you want to overwrite it?",
															  "File already Exists",
															  JOptionPane.YES_NO_OPTION,
															  JOptionPane.QUESTION_MESSAGE,
															  null, //don't use a custom Icon
															  options, //the titles of buttons
															  options[0]); //default button title
					//Overwrite the file
					if (cancel == 0){
						saveBoardToFile(fileName);
					}
				} catch (IOException ex) {
					System.out.println("File doesn't exist creating it.. ");
					saveBoardToFile(fileName);
				} // end of try / catch for file acces errors
			}
		}


    }


    private boolean isDragging = false;
    private KeyFrame dragedKeyFrame;
    private int dragedKeyFramePos;
    private int deletKeyFramePos;
    public boolean frameIsClicked = false;
    public boolean isDraggingNewFrame = false;

    /**
     * The Event Handler for any board-mouse interaction
     */
    class BoardEventHandler extends MouseAdapter {

        //True if one may draw on the board
        boolean isWritable = false;

        //Coonstructor
        public BoardEventHandler(boolean isWritable) {
            this.isWritable = isWritable;

        }

        public void mouseReleased(MouseEvent e) {
            System.out.println("Mouse was released");
            isDragging = false;

            if (isDraggingNewFrame){

                int frameNumber = e.getX();
                int value = e.getY();
                System.out.println("Mouse was released: FrameNumber = " + frameNumber + "  value = " + value);
                drawingBoard.addKeyFrame(actualDrawingMotor, frameNumber, value);
                resultBoard.addKeyFrame(actualDrawingMotor, frameNumber, value);
                jLabelServoFrameNumber.setText("Frame position:");
                jLabelServoFrameNumber2.setText(String.valueOf(frameNumber));
                jLabelServoValue.setText("Value:");
                jLabelServoValue2.setText(String.valueOf(drawingBoard.transformToServoValue(value)));
                jLabelServoId.setText("ServoId:");
                jLabelServoId2.setText(GlobalDefinitions.servosNames[actualDrawingMotor]);
                Cursor myCursor = new Cursor(Cursor.HAND_CURSOR);
                setCursor(myCursor);
                dragedKeyFramePos=drawingBoard.boardKeyFrames.vec.size();
            }
            isDraggingNewFrame=false;

        }

        public void mousePressed(MouseEvent e) {
            frameIsClicked = false;
            int frameNumber = e.getX();
            int value = e.getY();
            if (isWritable) {
                dragedKeyFramePos = drawingBoard.isOnKeyFrame(actualDrawingMotor, frameNumber, value);
                System.out.println("Mouse was Pressed: FrameNumber = " + frameNumber + "  value = " + value + " dragedKeyFramePos = " + dragedKeyFramePos);
                if (dragedKeyFrame.frameNumber >= 0) {
                    isDragging = true;
                    frameIsClicked = true;
                }
                jButtonDelete.setEnabled(false);
            }
        }

        /**
         * The eventHandler in case the mouse is clicked over the board:
         * If over a KeyFrame goto Edit Mode else create New Frame
         * @param e The Mouse event with it mouse related information
         */
        public void mouseClicked(MouseEvent e) {
            Cursor myCursor;
            if (isWritable && !frameIsClicked) {
                int frameNumber = e.getX();
                int value = e.getY();
                System.out.println("Mouse was clicked: FrameNumber = " + frameNumber + "  value = " + value);
                drawingBoard.addKeyFrame(actualDrawingMotor, frameNumber, value);
                resultBoard.addKeyFrame(actualDrawingMotor, frameNumber, value);
                jLabelServoFrameNumber.setText("Frame position:");
                jLabelServoFrameNumber2.setText(String.valueOf(frameNumber));
                jLabelServoValue.setText("Value:");
                jLabelServoValue2.setText(String.valueOf(drawingBoard.transformToServoValue(value)));
                jLabelServoId.setText("ServoId:");
                jLabelServoId2.setText(GlobalDefinitions.servosNames[actualDrawingMotor]);

                myCursor = new Cursor(Cursor.HAND_CURSOR);
                setCursor(myCursor);
            } else if (isWritable && frameIsClicked) {
                System.out.println("You clicked over a frame");
                jButtonDelete.setEnabled(true);
                deletKeyFramePos = dragedKeyFramePos;
            }
            isDragging = false;
        }


    }


    /**
     * The Event Handler for any board-mouse interaction
     */
    class BoardMotionEventHandler extends MouseMotionAdapter {

        //True if one may draw on the board
        boolean isWritable = false;


        //Coonstructor
        public BoardMotionEventHandler(boolean isWritable) {
            this.isWritable = isWritable;

        }

        public void mouseDragged(MouseEvent e) {

            if (isDragging) {

                int x = e.getX();
                int y = e.getY();
                if (x < 0) x = 0;
                if (x > drawingBoard.boardWidth - 1) x = drawingBoard.boardWidth - 1;

                if (y < 0) y = 0;
                if (y > drawingBoard.boardHeight - 1) y = drawingBoard.boardHeight - 1;

                if (y < drawingBoard.maxBoardValue) y = drawingBoard.maxBoardValue;
                if (y > drawingBoard.minBoardValue) y = drawingBoard.minBoardValue;

                System.out.println("Frame is beeing dragged e.x = " + x + " e.y = " + y);
                drawingBoard.moveKeyframe(actualDrawingMotor, dragedKeyFramePos, x, y);
                resultBoard.moveKeyframe(actualDrawingMotor, dragedKeyFramePos, x, y);
                jLabelServoFrameNumber.setText("Frame Position:");
                jLabelServoFrameNumber2.setText(String.valueOf(x));
                jLabelServoValue.setText("Value:");
                jLabelServoValue2.setText(String.valueOf(drawingBoard.transformToServoValue(dragedKeyFrame.servoValue)));
                jLabelServoId.setText("ServoId:");
                jLabelServoValue2.setText(String.valueOf(drawingBoard.transformToServoValue(y)));

            }else{


                isDraggingNewFrame = true;


            }

        }

        public void mouseMoved(MouseEvent e) {

            int frameNumber = e.getX();
            int value = e.getY();
            jLabelMousePosX.setText("FrameNumber:  " + String.valueOf(frameNumber));
            jLabelMousePosY.setText("ServoValue:  " + String.valueOf(value));

            dragedKeyFrame = drawingBoard.getKeyFrameUnderMouse(actualDrawingMotor, frameNumber, value);
            //Is under a frame
            if (dragedKeyFrame.frameNumber < 0) {
                jLabelServoFrameNumber.setText("");
                jLabelServoValue.setText("");
                jLabelServoId.setText("");
                jLabelServoFrameNumber2.setText("");
                jLabelServoValue2.setText("");
                jLabelServoId2.setText("");
            } else {
                jLabelServoFrameNumber.setText("Frame Position:");
                jLabelServoFrameNumber2.setText(String.valueOf(dragedKeyFrame.frameNumber));
                jLabelServoValue.setText("Value:");
                jLabelServoValue2.setText(String.valueOf(drawingBoard.transformToServoValue(dragedKeyFrame.servoValue)));
                jLabelServoId.setText("ServoId:");
                jLabelServoId2.setText(GlobalDefinitions.servosNames[actualDrawingMotor]);

            }


            dragedKeyFramePos = drawingBoard.isOnKeyFrame(actualDrawingMotor, frameNumber, value);
            Cursor myCursor;

            if (dragedKeyFramePos >= 0 && isWritable)
                myCursor = new Cursor(Cursor.HAND_CURSOR);


            else
                myCursor = new Cursor(Cursor.DEFAULT_CURSOR);
            setCursor(myCursor);

            //System.out.println("Mouse is moving e.x = " + e.getX() + " e.y = " + e.getY() + " is over Frame = " + position);

        }

    }

    /**
     * initiates the Robot Buttons and adds them to the interface
     */

//The two Button Mouse listeners
    DrawingBoardButtonsEventHandler drawingButtonsHandler = new DrawingBoardButtonsEventHandler(this);
    ResultBoardButtonsEventHandler resultButtonsHandler = new ResultBoardButtonsEventHandler();

//The robot toggle buttons
    ButtonVector drawingButtons = new ButtonVector(drawingButtonsHandler);
    ButtonVector resultButtons = new ButtonVector(resultButtonsHandler);

    
    private void initRobotButtons() {
	
        //Define the Board/Motor Buttons
	
        //Drawing Board
        drawingButtons.addButton("0", new Insets(1, 1, 1, 1), new Rectangle(130, 67, 22, 17), true);
		drawingButtons.addButton("1", new Insets(1, 1, 1, 1), new Rectangle(119, 85, 22, 17), false);
        drawingButtons.addButton("2", new Insets(1, 1, 1, 1), new Rectangle(97, 85, 22, 17), false);
        drawingButtons.addButton("3", new Insets(1, 1, 1, 1), new Rectangle(75, 85, 22, 17), false);
        drawingButtons.addButton("4", new Insets(1, 1, 1, 1), new Rectangle(97, 102, 22, 17), false);
        drawingButtons.addButton("5", new Insets(1, 1, 1, 1), new Rectangle(86, 119, 22, 17), false);
        drawingButtons.addButton("6", new Insets(1, 1, 1, 1), new Rectangle(108, 119, 22, 17), false);
		drawingButtons.addButton("7", new Insets(1, 1, 1, 1), new Rectangle(141, 85, 22, 17), false);
        drawingButtons.addButton("8", new Insets(1, 1, 1, 1), new Rectangle(163, 85, 22, 17), false);
        drawingButtons.addButton("9", new Insets(1, 1, 1, 1), new Rectangle(185, 85, 22, 17), false);
        drawingButtons.addButton("10", new Insets(1, 1, 1, 1), new Rectangle(163, 102, 22, 17), false);
        drawingButtons.addButton("11", new Insets(1, 1, 1, 1), new Rectangle(152, 119, 22, 17), false);
        drawingButtons.addButton("12", new Insets(1, 1, 1, 1), new Rectangle(174, 119, 22, 17), false);
        drawingButtons.addButton("13", new Insets(1, 1, 1, 1), new Rectangle(119, 49, 22, 17), false);
        drawingButtons.addButton("14", new Insets(1, 1, 1, 1), new Rectangle(97, 49, 22, 17), false);
		drawingButtons.addButton("15", new Insets(1, 1, 1, 1), new Rectangle(75, 49, 22, 17), false);
        drawingButtons.addButton("16", new Insets(1, 1, 1, 1), new Rectangle(53, 42, 22, 17), false);
		drawingButtons.addButton("17", new Insets(1, 1, 1, 1), new Rectangle(31, 35, 22, 17), false);
		drawingButtons.addButton("18", new Insets(1, 1, 1, 1), new Rectangle(141, 49, 22, 17), false);
        drawingButtons.addButton("19", new Insets(1, 1, 1, 1), new Rectangle(163, 49, 22, 17), false);
		drawingButtons.addButton("20", new Insets(1, 1, 1, 1), new Rectangle(185, 49, 22, 17), false);
        drawingButtons.addButton("21", new Insets(1, 1, 1, 1), new Rectangle(207, 42, 22, 17), false);
		drawingButtons.addButton("22", new Insets(1, 1, 1, 1), new Rectangle(229, 35, 22, 17), false);
        drawingButtons.addButton("23", new Insets(1, 1, 1, 1), new Rectangle(119, 30, 22, 17), false);
		drawingButtons.addButton("24", new Insets(1, 1, 1, 1), new Rectangle(141, 30, 22, 17), false);

		//Result Board
        resultButtons.addButton("0", new Insets(1, 1, 1, 1), new Rectangle(370, 67, 22, 17), false);
		resultButtons.addButton("1", new Insets(1, 1, 1, 1), new Rectangle(359, 85, 22, 17), false);
        resultButtons.addButton("2", new Insets(1, 1, 1, 1), new Rectangle(337, 85, 22, 17), false);
        resultButtons.addButton("3", new Insets(1, 1, 1, 1), new Rectangle(315, 85, 22, 17), false);
        resultButtons.addButton("4", new Insets(1, 1, 1, 1), new Rectangle(337, 102, 22, 17), false);
        resultButtons.addButton("5", new Insets(1, 1, 1, 1), new Rectangle(326, 119, 22, 17), false);
        resultButtons.addButton("6", new Insets(1, 1, 1, 1), new Rectangle(348, 119, 22, 17), false);
		resultButtons.addButton("7", new Insets(1, 1, 1, 1), new Rectangle(381, 85, 22, 17), false);
        resultButtons.addButton("8", new Insets(1, 1, 1, 1), new Rectangle(403, 85, 22, 17), false);
        resultButtons.addButton("9", new Insets(1, 1, 1, 1), new Rectangle(425, 85, 22, 17), false);
        resultButtons.addButton("10", new Insets(1, 1, 1, 1), new Rectangle(403, 102, 22, 17), false);
        resultButtons.addButton("11", new Insets(1, 1, 1, 1), new Rectangle(392, 119, 22, 17), false);
        resultButtons.addButton("12", new Insets(1, 1, 1, 1), new Rectangle(414, 119, 22, 17), false);

        resultButtons.addButton("13", new Insets(1, 1, 1, 1), new Rectangle(359, 49, 22, 17), false);
        resultButtons.addButton("14", new Insets(1, 1, 1, 1), new Rectangle(337, 49, 22, 17), false);
		resultButtons.addButton("15", new Insets(1, 1, 1, 1), new Rectangle(315, 49, 22, 17), false);
        resultButtons.addButton("16", new Insets(1, 1, 1, 1), new Rectangle(293, 42, 22, 17), false);
		resultButtons.addButton("17", new Insets(1, 1, 1, 1), new Rectangle(271, 35, 22, 17), false);
             
        resultButtons.addButton("18", new Insets(1, 1, 1, 1), new Rectangle(381, 49, 22, 17), false);
        resultButtons.addButton("19", new Insets(1, 1, 1, 1), new Rectangle(403, 49, 22, 17), false);
		resultButtons.addButton("20", new Insets(1, 1, 1, 1), new Rectangle(425, 49, 22, 17), false);
        resultButtons.addButton("21", new Insets(1, 1, 1, 1), new Rectangle(447, 42, 22, 17), false);
        resultButtons.addButton("20", new Insets(1, 1, 1, 1), new Rectangle(469, 35, 22, 17), false);
        resultButtons.addButton("23", new Insets(1, 1, 1, 1), new Rectangle(359, 30, 22, 17), false);
		resultButtons.addButton("24", new Insets(1, 1, 1, 1), new Rectangle(381, 30, 22, 17), false);

//Add the Motor Buttons to the panel
        for (int i = 0; i < drawingButtons.buttonVec.size(); i++) {
            this.getContentPane().add(drawingButtons.buttonAt(i), null);
            this.getContentPane().add(resultButtons.buttonAt(i), null);
        }

    }

    public void unselectAllButtons(int buttonSelected) {
        //Unselect all Buttons
        Iterator it = drawingButtons.buttonVec.iterator();
        int i = 0;
        while (it.hasNext()) {
            ((JToggleButton) it.next()).setSelected(false);
            resultBoard.isVisible[i] = true;
            i++;
        }
//Set the focus to the Button clicked on
        drawingButtons.buttonAt(buttonSelected).setSelected(true);
    }

//Event Handler for board toggle buttons
    class DrawingBoardButtonsEventHandler extends MouseAdapter {
        MotionInterface interFace;

        public DrawingBoardButtonsEventHandler(MotionInterface interFace) {
            this.interFace = interFace;

        }

        public void mouseClicked(MouseEvent e) {

            int buttonId = Integer.parseInt(((JToggleButton) e.getSource()).getText());
            System.out.println("Drawing Board Button " + buttonId + " clicked!!");
            //parent.displayMotor[buttonId] = 1;

            unselectAllButtons(buttonId);

            //Get the actual drawing and put it in the motion Vector
            ServoKeyFrames newDrawing = drawingBoard.getTransFormedServoKeyFrames(actualDrawingMotor);
            System.out.println("Saving Drawing...: " + newDrawing.toString());
            actualMotion.appendServoKeyFrames(newDrawing);
            //change to the servo slected
            actualDrawingMotor = buttonId;

            //Search in the actual Motion if the servo already has keyFrames
            ServoKeyFrames newMotion = actualMotion.getServoKeyFrames(actualDrawingMotor);
            System.out.println("New Loaded from Motion: " + newMotion.toString());

            //Transform the new Motion's keyFrames to drawing Board values and replace the boards key Frames
            drawingBoard.setTransformedServoKeyFrames(newMotion, false);
            //resultBoard.boardKeyFrames.setKeyFrames(temp, true);
            System.out.println("Is the actual one : " + drawingBoard.boardKeyFrames.getServoKeyFrames(actualDrawingMotor).toString());
            //Set the servos limits to the board
            drawingBoard.setMinMaxBoardValuesFromServoLimits(
				GlobalDefinitions.servoMinValue[actualDrawingMotor], 
				GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
            resultBoard.setMinMaxBoardValuesFromServoLimits(
				GlobalDefinitions.servoMinValue[actualDrawingMotor], 
				GlobalDefinitions.servoMaxValue[actualDrawingMotor]);
            drawingBoard.redrawBoard();
            resultBoard.redrawBoard();


            //interFace.repaint();

            //Change back to default
            /*parent.jCheckBoxOne.setSelected(true);
			  parent.jCheckBoxAll.setSelected(false);
			  parent.CopyPasteTarget = 0;
			  parent.redrawBoard();
            */
        }

    }

//Event Handler for result toggle buttons
    class ResultBoardButtonsEventHandler extends MouseAdapter {

        public ResultBoardButtonsEventHandler() {
        }

        public void mouseClicked(MouseEvent e) {
            JToggleButton clickedButton = (JToggleButton) e.getSource();
            int buttonId = Integer.parseInt(clickedButton.getText());
            System.out.println("Result Board Button " + buttonId + " clicked!!");

            if (clickedButton.isSelected()) {
                System.out.println("Was selcted.. changing...");
                //resultButtons.buttonAt(buttonId).setEnabled(false);
                resultBoard.isVisible[buttonId] = false;
            } else {
                System.out.println("Was not selcted.. changing...");
                // resultButtons.buttonAt(buttonId).setEnabled(true);
                resultBoard.isVisible[buttonId] = true;
            }
            resultBoard.redrawBoard();

        }

    }
}
