package MotionTool;

import java.io.*;
import com.cyberbotics.webots.Controller;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;

public class Motion {

    //Motion Name
    public String motionName = "";
    public int motionID;
 
    //Frames
    public int numberOfFramesInMotion = 0;
    public int motionStartedAtFrame = 0;
    public int actualFrame = 0;
    private int numberOfSmoothenFrames = 0;
    public Vector motionDefinition = new Vector(10, 5); //vector of ServoKeyFrames
    private Vector numberOfFramesOfservos= new Vector(10,5);
    //private int startFrame =0;
    //private int positionOfStartFrameInMotion = 0; //Keeps track of the begining of the actual frame

    //Default parameters
    public boolean reverse = false;
    public boolean inversed = false;
    public int loop = 1;
    public int AIMessage;
    public float smoothFactorMotionTransition = 0;
    public Vector servoValuesAtStart = new Vector(22);

    //Constructor
    public Motion() {
    }

    public int getDisplayTime(){

        if (actualFrame < 0) return 0;
       return actualFrame;
    }
    //Initialise the motion for current frame
    public void initMotion(int actualFrame) {

        servoValuesAtStart.clear();
        //Init the startframe (relative begining of Motion)
        motionStartedAtFrame = actualFrame;
        //Store the initialValues of the Servos concerned by the motion
        Enumeration enumServos = motionDefinition.elements();
        while (enumServos.hasMoreElements()) {
            int servoId = ((ServoKeyFrames) enumServos.nextElement()).servoId;
            servoValuesAtStart.add(new Float(Controller.servo_get_position(servoId)));
        }

         System.out.println("Initialising Motion smoothFactor is :" + smoothFactorMotionTransition);
        if (smoothFactorMotionTransition > 0.0f){
            //Add interpolation frames to smothen the change from the actual
            //position to the begining of this motion. The maximal speed for
            //the change is defined by the smoothFactorMotionTransition variable
            //This variables defines the number of frames needed to smoothen a change
            //of position equal to 1.0. The maximal alowed speed for position changes is
            //then derived from it : maxSpeed = 1.0/smoothFactorMotionTransition

            float maxDiffValue = 0;
            float maxSpeed = 0;
            float maxSpeedDiffValue = 0;
            float maxSpeedFrameNumber = 0;
            boolean hasZeroFrame = false;

            //Find the servo with the highest changing speed
            Enumeration enumServoKeyFrames = motionDefinition.elements();
            while (enumServoKeyFrames.hasMoreElements()) {

                //Get actual Servo's key frames
                ServoKeyFrames actualServoKeyFrame = (ServoKeyFrames) (enumServoKeyFrames.nextElement());
                //Store the initialValues of the Servos concerned by the motion
                float actualPositionOfServo = Controller.servo_get_position(actualServoKeyFrame.servoId);
                System.out.println("Actual servo Positio= "+ actualPositionOfServo);
                servoValuesAtStart.add(new Float(actualPositionOfServo));

                //If a keyFrame has a framenumber value equal to zero, we only store the diference
                //between the servo positions and look for the servo with the biggest one.
                KeyFrame firstKeyFrame = (KeyFrame) actualServoKeyFrame.keyFrames.firstElement();
                if (firstKeyFrame.frameNumber == 0) {
                    hasZeroFrame = true;
                    float tempDiff = Math.abs(firstKeyFrame.servoValue - actualPositionOfServo);
                    if (tempDiff > maxDiffValue) {
                        maxDiffValue = tempDiff;
                    }
                }
                //Else we look for the frame with the highest changing speed (Defined by the
                //difference between the actual pos (frame 0) and the first defined position of movement)
                else {
                    System.out.println("First Frame is not zero !!! firstFrame = " +firstKeyFrame.frameNumber + "actual position of Servo is "+actualPositionOfServo );
                    float m = (firstKeyFrame.servoValue - actualPositionOfServo) / firstKeyFrame.frameNumber;
                    if (Math.abs(m) > Math.abs(maxSpeed)) {
                        maxSpeed = m;
                        maxSpeedDiffValue = Math.abs(firstKeyFrame.servoValue - actualPositionOfServo);
                        maxSpeedFrameNumber = firstKeyFrame.frameNumber;
                    }

                }
            } //End while

            //Add corect number of frames to each servo in order to keep the slope
            //smaller then 1/smoothFactorMotionTransition
            //Example: if smoothFactorMotionTransition = 30 and the biggest diffrence
            //of servoPositions = 1 the slope must be smaler then 1/30 (30 Frames for
            //a change of servo position of 1
            float smoothSpeed = 1 / smoothFactorMotionTransition;
            boolean reCalcNumberOfSmoothenFrames = false;
            //One Keyframe has a frame number value equal to zero -> add frames to lower
            //the changing speed of the servo to the smooth speed
            if (hasZeroFrame) {
                numberOfSmoothenFrames = (int) (maxDiffValue / smoothSpeed);
                float newMaxSpeed1 = maxDiffValue/numberOfSmoothenFrames;
                float newMaxSpeed2 = maxSpeedDiffValue / (maxSpeedFrameNumber + numberOfSmoothenFrames);
                System.out.println("Max Speed 1 = "+ newMaxSpeed1 + " Max Speed 2 = "+newMaxSpeed2);
                //A Keyframe with frame number value bigger then zero must be used for smoothing
                if (newMaxSpeed2 > newMaxSpeed1) reCalcNumberOfSmoothenFrames = true;
            }

            //add frames to lower the changing speed of the servo to the smooth speed
            if (!hasZeroFrame || reCalcNumberOfSmoothenFrames) {
                if (smoothSpeed < maxSpeed) {
                    numberOfSmoothenFrames = (int) ((maxSpeedDiffValue / smoothSpeed) - maxSpeedFrameNumber);
                }
               System.out.println("First Frame is not zero !!! maxSpeedDiffValue = " +maxSpeedDiffValue + "smoothSpeed =  "+smoothSpeed+ " maxSpeedFrameNumber = "+ maxSpeedFrameNumber);
            }
            motionStartedAtFrame += numberOfSmoothenFrames;

        }//End if (smoothFactorMotionTransition > 0)

        System.out.println("Number of smoothen = " + numberOfSmoothenFrames);
    }


    public Vector getInterpolatedValues(int globalFrame) {


        //The result of this function
        Vector interPoltatedValues = new Vector();  //Vector of ServoControl's
        //The actualFrame relative to the motion
        actualFrame = globalFrame - motionStartedAtFrame;
        System.out.println("");
        System.out.println("Interpolation of "+ motionName + "  started at frame = " + motionStartedAtFrame);
        System.out.println("GlobalFrame = "+ globalFrame + " actualFrame = " + actualFrame + " Number of frames = " + numberOfFramesInMotion);

        //The motion is finished -> return empty vector
        if (actualFrame > numberOfFramesInMotion && numberOfFramesInMotion != 0) return new Vector();

        //Loop through all servos assinged by movement
        int numberOfServosInvolved = motionDefinition.size();
        for (int i = 0; i < numberOfServosInvolved; i++) {

            //actual ServoKeyFrame
            ServoKeyFrames actualServoKeyFrames = (ServoKeyFrames) motionDefinition.elementAt(i);
            int actualServoId = actualServoKeyFrames.servoId;
            //all the key Frames for the actual servo
            Vector keyFrames = actualServoKeyFrames.keyFrames;
            System.out.println("Servo Number " + actualServoId + " has " + keyFrames.size() +  "KeyFrames");
            if (keyFrames.size()>0){
                //Find start/end Key frames with corresponding values for actual Frame
                float startFramePositionInMotion = ((KeyFrame) keyFrames.elementAt(0)).frameNumber;
                float value=0f;

                //only interpolate from startPosition !!
                if (keyFrames.size()== 1 || actualFrame < 0 ){

                    //We are smoothening the transition from one motion to the other
                    if (actualFrame < 0 || actualFrame < startFramePositionInMotion){

                        float dy = ((KeyFrame)keyFrames.elementAt(0)).servoValue - ((Float)servoValuesAtStart.elementAt(i)).floatValue();
                        float dx =  (float)numberOfSmoothenFrames + startFramePositionInMotion;
                        value = dy / dx * (actualFrame + numberOfSmoothenFrames) + (((Float)servoValuesAtStart.elementAt(i)).floatValue());
                        System.out.println("Smoothening movemen, value calculated = "+ value + " Number of smoothenfarmes = " + numberOfSmoothenFrames + "dy = "+ dy + " dx = " + dx);

                    }else if (startFramePositionInMotion == 0){
                       value = ((KeyFrame)keyFrames.elementAt(0)).servoValue;
                         System.out.println("Key Frame size()=1 and position = 0, value calculated = "+ value);

                    }else{
                        float dy = ((KeyFrame)keyFrames.elementAt(0)).servoValue - (((Float)servoValuesAtStart.elementAt(i))).floatValue();
                        float dx =  (float)numberOfSmoothenFrames + startFramePositionInMotion;
                        value = dy / dx * (actualFrame + numberOfSmoothenFrames) + (((Float)servoValuesAtStart.elementAt(i))).floatValue();
                        System.out.println("Key Frame size()=1 and position > 0, dy = "+ dy + " dx = " + dx + " value calculated = "+ value + " Number of smoothenfarmes = " + numberOfSmoothenFrames);
                    }
                 //Find actual start/end Frame and interpolate relative to actualFrame
                }else if (actualFrame > startFramePositionInMotion){

                    float endFramePositionInMotion = startFramePositionInMotion + ((KeyFrame) keyFrames.elementAt(1)).frameNumber;
                    int endFramePositionInVector = 1;

                    //Find the position relative to the motion of the end Key Frame
                    int nbrOfKeyFrame = keyFrames.size();
                    while (actualFrame > endFramePositionInMotion && nbrOfKeyFrame > endFramePositionInVector+1) {

                        endFramePositionInVector++;
                        int oldnumberOfFramesInMotion = ((KeyFrame) keyFrames.elementAt(endFramePositionInVector)).frameNumber;
                        endFramePositionInMotion += oldnumberOfFramesInMotion;

                    }

                    int numberOfFramesInMotionOfEndFrame = ((KeyFrame) keyFrames.elementAt(endFramePositionInVector)).frameNumber;
                    //The start Frame position is the end position minus the number of frames of end Key Frame
                    startFramePositionInMotion = endFramePositionInMotion - ((KeyFrame) keyFrames.elementAt(endFramePositionInVector)).frameNumber;
                    float actualStartValue = ((KeyFrame) keyFrames.elementAt(endFramePositionInVector - 1)).servoValue;
                    float actualEndValue = ((KeyFrame) keyFrames.elementAt(endFramePositionInVector)).servoValue;


                        //Interpolate servo value
                        float deltaY = actualEndValue - actualStartValue;
                        System.out.println("delta y = " + deltaY);
                        float m = deltaY / numberOfFramesInMotionOfEndFrame;
                        System.out.println("m = " + m);
                        value = m * (actualFrame - startFramePositionInMotion) + actualStartValue;
                        System.out.println("value = " + value);


                }//End else if
                else{
                    //The motion has started but we havent reached the first KeyFrame yet
                    System.out.println("Not reached first kexyFrame yet:");
                    System.out.println("First Frame is at : "+ startFramePositionInMotion);
                    System.out.println("Smooth Factor: "+ smoothFactorMotionTransition);
                    if (smoothFactorMotionTransition > 0){
                        float deltaY = (((KeyFrame) keyFrames.elementAt(0)).servoValue)- ((Float)servoValuesAtStart.elementAt(0)).floatValue();
                         System.out.println("delta y = " + deltaY);
                        float deltaX = numberOfSmoothenFrames + startFramePositionInMotion;
                        System.out.println("delta x = " + deltaX);
                        float m = deltaY/deltaX;
                        System.out.println("m = " + m);
                        value = m*(actualFrame+numberOfSmoothenFrames)+ ((Float)servoValuesAtStart.elementAt(0)).floatValue();
                        System.out.println("value = " + value);
                    }
                    else {
                        System.out.println("No smoothening!!");
                        value = ((Float)servoValuesAtStart.elementAt(0)).floatValue();
                    }
                }

                if (actualServoKeyFrames.numberOfFrames > actualFrame){
                    //Create return object (Servo Control) and add it to the result vector
                    ServoControl temp = new ServoControl();
                    temp.servoId = actualServoId;
                    temp.value = value;

                    interPoltatedValues.add(temp);

                }
            }// End if KeyFrame size > 0

        }//End for loop

        return interPoltatedValues;
    }

    //public float interpolate (float startValue, float endValue, int startFrame, int endFrame, int actualFrame)
    /**
     * Setting the SmoothFactorMotionTransition :
     * Its usefull for avoid the robot falling while changing motions
     * When the robot change its motion, the seqManager check the biggest difference value for each servos (exept the neck servo)
     * from the current postion to next postion of the incoming motion
     * Then it add to the interpolation value of the first keyframe of the next motion: difference*SmoothFactorMotionTransition
     * Sample : if the current position of the Left_Hip1 is -1.0 and 2.0 for the next motion, and if SmoothFactorMotionTransition is 10
     * then it will add 3.0*10= 30 interpolated frames for smooth the transition from the old motion to the next motion
     * Sometime, it not enought to avoid the robot falling, so you'll then have to add a 'safe motion' inside 2 extremes motions
     */
    public boolean setSmoothFactorMotionTransition(float Factor) {
        smoothFactorMotionTransition = Factor;
        return true;
    }

    //Add a KeyFrame to the motion
    public void addKeyFrame(int servoId, int frameNumber, float servoValue) {

        //If there already exists a ServoKeyFrame with the actual servoId, append actual KeyFrame to it
        Enumeration enumKeyFrames = motionDefinition.elements();
        while (enumKeyFrames.hasMoreElements()) {
            ServoKeyFrames frame = (ServoKeyFrames) enumKeyFrames.nextElement();
            //Servo has already a ServoiKeyFrame -> add New one
            if (frame.servoId == servoId) {
                frame.keyFrames.add(new KeyFrame(frameNumber, servoValue));
                return;
            }
        }

        //Else create a new ServoKeyFrame and add actual KeyFrame to it
        KeyFrame frame = new KeyFrame(frameNumber, servoValue);
        ServoKeyFrames servoFrame = new ServoKeyFrames(servoId, frame);
        motionDefinition.add(servoFrame);
    }



    public void appendServoKeyFrames(ServoKeyFrames keyFrames){

        int servoId = keyFrames.servoId;
        //If there already exists a ServoKeyFrame with the actual servoId
        //remove it. Add new ServoKeyFrames to motionDefinition
        for (int i =0; i < motionDefinition.size(); i++){

            if(servoId ==((ServoKeyFrames)motionDefinition.elementAt(i)).servoId){
                //Servo has already a ServoiKeyFrame -> remove it
                motionDefinition.removeElementAt(i);
                break;
            }
        }
        motionDefinition.addElement(keyFrames);
    }

    public ServoKeyFrames getServoKeyFrames(int servoId){
        Iterator it= motionDefinition.iterator();
        int pos=0;
        while (it.hasNext()){
            if (((ServoKeyFrames)it.next()).servoId == servoId) return (ServoKeyFrames)motionDefinition.elementAt(pos);
            pos++;
        }

        return new ServoKeyFrames(servoId);
    }


    //Finds and stores the number of frames of the longest servo sequence
    public void updatenumberOfFramesInMotion() {
        numberOfFramesInMotion=0;
        Enumeration enumServoKeyFrames = motionDefinition.elements();
        while (enumServoKeyFrames.hasMoreElements()) {
            int frameNumberOfServo= ((ServoKeyFrames)enumServoKeyFrames.nextElement()).upDateNumberOfFrames();
            if (frameNumberOfServo > numberOfFramesInMotion) numberOfFramesInMotion = frameNumberOfServo;
        }
    }

    public String toString() {

        String result = "Motion " + motionName + "(" + motionID + "): \n";
        Enumeration enumServoKeyFrames = motionDefinition.elements();
        while (enumServoKeyFrames.hasMoreElements()) {

            ServoKeyFrames temp = ((ServoKeyFrames) enumServoKeyFrames.nextElement());
            result += "\nServo " + temp.servoId + "\n";
            Enumeration enumKeyFrames = temp.keyFrames.elements();

            while (enumKeyFrames.hasMoreElements()) {

                KeyFrame tempKey = (KeyFrame) enumKeyFrames.nextElement();
                int actualFrameNumber = tempKey.frameNumber;
                float actualServoValue = tempKey.servoValue;
                result += " Frame " + actualFrameNumber + " , Value  " + actualServoValue + "\n";
            }
        }
        return result;
    }



    //Returned to caller of getInterpolationValues
    public class ServoControl {
        public int servoId;
        public float value;

    }
}
