/****************************************************************
 *                                                              *
 *   Biped Locomotion                                           *
 *                                                              *
 *   Created by:  Jean-Phillipe Eager                           *
 *   Adapted by:  Pascal Cominoli (pascal.cominoli@epfl.ch)     *
 *                Yariv Bachar (Y.Bachar@sms.ed.ac.uk)          *
 *                                                              *
 ****************************************************************
 * $Id: $
 ****************************************************************/
/*
 * File: hoap2_motionTool.java
 * Description: Controller for the hoap2_motionTool.wbt webots 
 * world, implementing a motion planning tool.
 */

import com.cyberbotics.webots.Controller;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.lang.*;
import java.util.Vector;
import java.util.Enumeration;
import MotionTool.Interface.*;
import MotionTool.Motion;
import MotionTool.FileParser;
import MotionTool.Sequence;

/** ----------------------------------------------------------------------------
 * @class hoap2_motionTool                                                    
 * @brief Implements the Webots controller for hoap2 model with GUI motion 
 * planning tool.
 * -------------------------------------------------------------------------- */
public abstract class hoap2_motionTool extends Controller {
    static int actualFrame = 0;
    static int nbrJoints = 25;
    static int i, j, k, l;
    static int[] joint = new int[nbrJoints];
    static int camera;
    static int distance;
    static int left_touch;
    static int right_touch;
    static float gpsx, gpsy, gpsz;
    static int gps;
    static int[] image;
    static float[] matrix;
    static int emitter;
    static int front_led;
    static String str = "";
    // load the interface
    static MotionInterface motionInterface = new MotionInterface();

    public static void die() 
	{
		System.out.println("die method called");
    }

    // Reset all the neccesary variables wenn the stop button was hit
    public static void initializeVariables() 
	{
        // Sets all the motors to 0:
        for (int i = 0; i < 10; i++) {
            for (int no = 0; no < nbrJoints; no++) {
                servo_set_position(joint[no], 0.0f);
            }
            robot_step(32);
        }
        // Initialization of variables
        actualFrame = 0;
        motionInterface.letsGo = 0;

        // Resets the interface
        motionInterface.panelTimeBar.setVisible(false);
        motionInterface.jButtonGo.setBackground(Color.green);
        motionInterface.jButtonGo.setText("GO");
    }

    public static void reset() 
	{
		System.out.println("starting reset\n");
		joint[0] = robot_get_device("body_joint_1");
		joint[1] = robot_get_device("lleg_joint_1");
		joint[2] = robot_get_device("lleg_joint_3");
		joint[3] = robot_get_device("lleg_joint_2");
		joint[4] = robot_get_device("lleg_joint_4");
		joint[5] = robot_get_device("lleg_joint_5");
		joint[6] = robot_get_device("lleg_joint_6");
		joint[7] = robot_get_device("rleg_joint_1");
		joint[8] = robot_get_device("rleg_joint_3");
		joint[9] = robot_get_device("rleg_joint_2");
		joint[10] = robot_get_device("rleg_joint_4");
		joint[11] = robot_get_device("rleg_joint_5");
		joint[12] = robot_get_device("rleg_joint_6");
		joint[13] = robot_get_device("larm_joint_1");
		joint[14] = robot_get_device("larm_joint_2");
		joint[15] = robot_get_device("larm_joint_3");
		joint[16] = robot_get_device("larm_joint_4");
		joint[17] = robot_get_device("larm_joint_5");
		joint[18] = robot_get_device("rarm_joint_1");
		joint[19] = robot_get_device("rarm_joint_2");
		joint[20] = robot_get_device("rarm_joint_3");
		joint[21] = robot_get_device("rarm_joint_4");
		joint[22] = robot_get_device("rarm_joint_5");
		joint[23] = robot_get_device("head_joint_2");
		joint[24] = robot_get_device("head_joint_1");
 
		for(int i = 0; i < nbrJoints; i++) 
			if (joint[i] == 0)
				System.out.println("Joint "+i+" is missing");
		
		camera = robot_get_device("camera");
		distance = robot_get_device("distance");
		left_touch = robot_get_device("left touch");
		right_touch = robot_get_device("right touch");
		gps = robot_get_device("gps");
		emitter = robot_get_device("emitter");
// 		front_led = robot_get_device("front eye");
    }
  
    public static void initInterface() 
	{
        motionInterface.setLocation(380, -10);
        motionInterface.setSize(640, 720);
        motionInterface.setVisible(true);
        motionInterface.initialize();
    }

    public static void main() 
	{
		float t = 0.0f;
		int[] image;
		int l;
		float f;
		float[] matrix;
		float x, y, z;

		// var for tricky
		float tr = 1.5f;
		float tr2 = 1.5f;

		String message = "stand me up 0 0.5 0.5 0";
		// the figures here mean the following:
		// 0: robot = Judoka0
		// 0.5 the X position of the robot
		// 0.5 the Z position of the robot
		// 0 the orientation of the robot (radians)
		// It is also possible to simply pass "stand me up 0"
		// the supervisor will then use the last initial position
		byte [] msg = null;
		int send = 0;

		try { // convert the String into bytes using the ISO-8859-1 encoding
			msg = message.getBytes("ISO-8859-1");
		} catch(Exception e) { }

		// Initialise some interface parameters:
		initInterface();
	    
		camera_enable(camera, 64);
		distance_sensor_enable(distance, 64);
		touch_sensor_enable(left_touch, 64);
		touch_sensor_enable(right_touch, 64);
		gps_enable(gps, 64);
		for (int i = 0; i < nbrJoints; i++) {
			servo_enable_position(joint[i], 32);
		}
		initializeVariables();
    
		// Initialize the actualMotion (is set to an empty motion):
		Motion actualMotion = motionInterface.getActualMotion();
		actualFrame = 0;
				
		for (;;) {				// The robot never dies!
			image = camera_get_image(camera);
			l = distance_sensor_get_value(distance);
			// System.out.println("Distance sensor: "+l);
			f = servo_get_position(joint[20]);
			// System.out.println("Neck position: "+f);
			l = touch_sensor_get_value(left_touch);
			//System.out.println("Left touch sensor: "+l);
			matrix = gps_get_matrix(gps); // OpenGL 4x4 matrix
			gpsx = gps_position_x(matrix);
			gpsy = gps_position_y(matrix);
			gpsz = gps_position_z(matrix);
		
			// will be done only once after reset
			if (motionInterface.letsGo == 1) {
				System.out.println("************************ Pressed Go...!! *********************");
				motionInterface.letsGo = 2;
				actualFrame = 0;
				actualMotion = motionInterface.getActualMotion();
//                 actualMotion.updatenumberOfFramesInMotion();
// 				actualMotion.setSmoothFactorMotionTransition(0);
				actualMotion.initMotion(actualFrame);
				System.out.println(actualMotion.toString());

				// puts the drawings at the beginning of the planning:
				motionInterface.periodBeginning = motionInterface.timeAtTheBeginning;
				actualFrame = motionInterface.timeAtTheBeginning;
				str = Integer.toString(motionInterface.periodBeginning);
// 				motionInterface.jTextBeginning.setText(str);
				motionInterface.periodEnd = motionInterface.periodBeginning + 
					motionInterface.resultBoard.boardWidth;
				str = Integer.toString(motionInterface.periodEnd);
// 				motionInterface.jTextEnd.setText(str);
// 				motionInterface.redrawBoard();

				// Displays the time bar:
				motionInterface.panelTimeBar.setVisible(true);
				motionInterface.panelTimeBar.setLocation(actualFrame, 170);
			}
			
			// The controller is running the actualMotion (actualFrame steers 
			// the position):
			if (motionInterface.letsGo == 2) {
				if (!motionInterface.jCheckBoxManualMode.isSelected()) {
					motionInterface.panelTimeBar.setLocation(
						actualMotion.getDisplayTime() + 5, 170);
					Vector servoPositions = 
						actualMotion.getInterpolatedValues(actualFrame);
					try {
						for (int i = 0; i < servoPositions.size(); i++) {
							Motion.ServoControl control = 
								(Motion.ServoControl) servoPositions.elementAt(i);
							System.out.println("Sending value " + control.value 
											   + " to servo " + control.servoId);
							servo_set_position(joint[control.servoId], control.value);
						}
					} catch (Exception e) {
						System.out.println(e.toString());
					}
					actualFrame++;
				}
			}

			// Restart if user has pushed the STOP button
			if (motionInterface.letsGo == 3) {
				System.out.println(
					"*************** Pressed STOP...!! ******************");
				emitter_send(emitter, msg.length, msg);
				send = 35;
				initializeVariables();
			}

			//  displays time bar
// 			if (actualFrame >= motionInterface.periodEnd) {
// 				motionInterface.periodBeginning = motionInterface.periodEnd;
// 				str = Integer.toString(motionInterface.periodBeginning);
// 				// motionInterface.jTextBeginning.setText(str);
// 				motionInterface.periodEnd = motionInterface.periodBeginning + 
// 					motionInterface.resultBoard.boardWidth;
// 				str = Integer.toString(motionInterface.periodEnd);
// 				// motionInterface.jTextEnd.setText(str);
// 			   // motionInterface.redrawBoard();
// 			}
// 			if (5 - motionInterface.periodBeginning + actualFrame > 0 && 
// 				5 - motionInterface.periodBeginning + actualFrame < 
// 				motionInterface.resultBoard.boardWidth) { // 5 is the nbr of pixels between frame and drawing
// 				motionInterface.panelTimeBar.setLocation(
// 					5 - motionInterface.periodBeginning + actualFrame, 223);
// 				motionInterface.panelTimeBar.repaint();
// 			}

			if (actualFrame == motionInterface.maxTime) {
				actualFrame = 0;
				actualMotion.initMotion(actualFrame);
			}
			
			robot_step(32); // run one step
		}
    }
}
