/* -*- c++ -*- */
/****************************************************************
 *                                                              *
 *  Project name:      Biped Locomotion                         *
 *                                                              *
 *  Created by:  Yariv Bachar (Y.Bachar@sms.ed.ac.uk)           *
 ****************************************************************
 * $Id: biped_gene.hpp,v 1.5 2004/08/06 18:02:42 s0343981 Exp $
 ****************************************************************/
/**
 * \file biped_gene.hpp 
 * \brief class Gene definition.
 */

#ifndef __BIPED_GENE_HPP__
#define __BIPED_GENE_HPP__

// =============================================================================
//  Class: Gene                                                                
/// \brief Data structure holding open-loop sinusoidal servo actuation. 
// =============================================================================
class Gene {
 public:
	Gene() : jointID_(0) , A_(1.4) , w_(1) , f_(0.0) , c_(0.0) { }
	Gene(int joint, double A, double w, double f, double c) : 
		jointID_(joint) , A_(A) , w_(w) , f_(f) , c_(c) { }
	double val(double t) const { return (A_ * sin(w_ * t + f_) + c_); }
	~Gene() { }

 public:
	int jointID_;				/**< the servo/joind ID */
	double A_;					/**< the amplitude of sinusoid */
	double w_;					/**< the frequency of sinusoid */
	double f_;					/**< the phase of sinusoid */
	double c_;					/**< the offset constant of sinusoid */
};

typedef Gene * GenePtr;

std::ostream & operator<<(std::ostream & out, const Gene & g)
{
	out << "\nGene: joint " << g.jointID_ << ", A = " << g.A_ << ", w = " 
		<< g.w_ << ", f = " << g.f_ << ", c = " << g.c_ << std::endl;
	return out;
}

#endif	// __BIPED_GENE_HPP__
